@@ -6,22 +6,14 @@
 
 import libtorrent as lt
 import time
+import sys
 
 ses = lt.session()
 ses.listen_on(6881, 6891)
 
-e = lt.bdecode(open("test.torrent", 'rb').read())
-info = lt.torrent_info(e)
-
-atp = {}
-atp["ti"] = info
-atp["save_path"] = "./"
-atp["storage_mode"] = lt.storage_mode_t(1)
-atp["paused"] = False
-atp["auto_managed"] = True
-atp["duplicate_is_error"] = True
-
-h = ses.add_torrent(atp)
+info = lt.torrent_info(sys.argv[1])
+h = ses.add_torrent({'ti': info, 'save_path': './'})
+print 'starting', h.name()
 
 while (not h.is_seed()):
 	s = h.status()
@@ -31,6 +23,9 @@ while (not h.is_seed()):
 	print '\r%.2f%% complete (down: %.1f kb/s up: %.1f kB/s peers: %d) %s' % \
 		(s.progress * 100, s.download_rate / 1000, s.upload_rate / 1000, \
 		s.num_peers, state_str[s.state]),
+	sys.stdout.flush()
 
 	time.sleep(1)
 
+print h.name(), 'complete'
+
