@@ -71,12 +71,30 @@ namespace
     torrent_info construct0(std::string path) {
         return torrent_info(fs::path(path));
     }
+
+    list map_block(torrent_info& ti, int piece, size_type offset, int size)
+    {
+       std::vector<file_slice> p = ti.map_block(piece, offset, size);
+       list result;
+
+       for (std::vector<file_slice>::iterator i(p.begin()), e(p.end()); i != e; ++i)
+          result.append(*i);
+
+       return result;
+    }
+
 } // namespace unnamed
 
 void bind_torrent_info()
 {
     return_value_policy<copy_const_reference> copy;
 
+    class_<file_slice>("file_slice")
+        .def_readwrite("file_index", &file_slice::file_index)
+        .def_readwrite("offset", &file_slice::offset)
+        .def_readwrite("size", &file_slice::size)
+        ;
+
     class_<torrent_info, boost::intrusive_ptr<torrent_info> >("torrent_info", no_init)
 #ifndef TORRENT_NO_DEPRECATE
         .def(init<entry const&>())
@@ -102,7 +120,9 @@ void bind_torrent_info()
 
         .def("num_files", &torrent_info::num_files, (arg("storage")=false))
         .def("file_at", &torrent_info::file_at, return_internal_reference<>())
+        .def("file_at_offset", &torrent_info::file_at_offset)
         .def("files", &files, (arg("storage")=false))
+        .def("rename_file", &torrent_info::rename_file)
 
         .def("priv", &torrent_info::priv)
         .def("trackers", range(begin_trackers, end_trackers))
@@ -113,6 +133,8 @@ void bind_torrent_info()
         .def("nodes", &nodes)
         .def("metadata", &metadata)
         .def("metadata_size", &torrent_info::metadata_size)
+        .def("map_block", map_block)
+        .def("map_file", &torrent_info::map_file)
         ;
 
     class_<file_entry>("file_entry")
@@ -124,6 +146,7 @@ void bind_torrent_info()
         )
         .def_readonly("offset", &file_entry::offset)
         .def_readonly("size", &file_entry::size)
+        .def_readonly("file_base", &file_entry::file_base)
         ;
 
     class_<announce_entry>("announce_entry", init<std::string const&>())
