@@ -229,7 +229,7 @@ std::string const& add_suffix(float val)
 	for (int i = 0; i < num_prefix; ++i)
 	{
 		val /= 1000.f;
-		if (fabs(val) < 1000.f)
+		if (std::fabs(val) < 1000.f)
 		{
 			std::string& ret = to_string(val, 4);
 			ret += prefix[i];
@@ -549,7 +549,7 @@ void scan_dir(path const& dir_path
 	for (directory_iterator i(dir_path), end; i != end; ++i)
 	{
 		if (extension(*i) != ".torrent") continue;
-		std::string file = i->string();
+		std::string file = i->path().string();
 
 		handles_t::iterator k = handles.find(file);
 		if (k != handles.end())
@@ -654,6 +654,13 @@ void handle_alert(libtorrent::session& ses, libtorrent::alert* a
 				ses.remove_torrent(h);
 		}
 	}
+	else if (save_resume_data_failed_alert* p = dynamic_cast<save_resume_data_failed_alert*>(a))
+	{
+		torrent_handle h = p->handle;
+		if (std::find_if(handles.begin(), handles.end()
+			, bind(&handles_t::value_type::second, _1) == h) == handles.end())
+			ses.remove_torrent(h);
+	}
 }
 
 static char const* state_str[] =
@@ -1132,6 +1139,12 @@ int main(int ac, char* av[])
 						::print_alert(holder.get(), std::cout);
 						std::cout << std::endl;
 
+						if (dynamic_cast<save_resume_data_failed_alert const*>(a))
+						{
+							--num_resume_data;
+							continue;
+						}
+
 						save_resume_data_alert const* rd = dynamic_cast<save_resume_data_alert const*>(a);
 						if (!rd) continue;
 						--num_resume_data;
