@@ -205,7 +205,8 @@ namespace libtorrent
 		{
 			static char const* state_str[] =
 				{"checking (q)", "checking", "dl metadata"
-				, "downloading", "finished", "seeding", "allocating"};
+				, "downloading", "finished", "seeding", "allocating"
+				, "checking (r)"};
 
 			return torrent_alert::message() + ": state changed to: "
 				+ state_str[state];
@@ -223,9 +224,9 @@ namespace libtorrent
 		tracker_error_alert(torrent_handle const& h
 			, int times
 			, int status
-			, std::string const& url
+			, std::string const& url_
 			, std::string const& msg_)
-			: tracker_alert(h, url)
+			: tracker_alert(h, url_)
 			, times_in_row(times)
 			, status_code(status)
 			, msg(msg_)
@@ -252,9 +253,9 @@ namespace libtorrent
 	struct TORRENT_EXPORT tracker_warning_alert: tracker_alert
 	{
 		tracker_warning_alert(torrent_handle const& h
-			, std::string const& url
+			, std::string const& url_
 			, std::string const& msg_)
-			: tracker_alert(h, url)
+			: tracker_alert(h, url_)
 			, msg(msg_)
 		{ TORRENT_ASSERT(!url.empty()); }
 
@@ -276,8 +277,8 @@ namespace libtorrent
 		scrape_reply_alert(torrent_handle const& h
 			, int incomplete_
 			, int complete_
-			, std::string const& url)
-			: tracker_alert(h, url)
+			, std::string const& url_)
+			: tracker_alert(h, url_)
 			, incomplete(incomplete_)
 			, complete(complete_)
 		{ TORRENT_ASSERT(!url.empty()); }
@@ -300,9 +301,9 @@ namespace libtorrent
 	struct TORRENT_EXPORT scrape_failed_alert: tracker_alert
 	{
 		scrape_failed_alert(torrent_handle const& h
-			, std::string const& url
+			, std::string const& url_
 			, std::string const& msg_)
-			: tracker_alert(h, url)
+			: tracker_alert(h, url_)
 			, msg(msg_)
 		{ TORRENT_ASSERT(!url.empty()); }
 
@@ -323,8 +324,8 @@ namespace libtorrent
 	{
 		tracker_reply_alert(torrent_handle const& h
 			, int np
-			, std::string const& url)
-			: tracker_alert(h, url)
+			, std::string const& url_)
+			: tracker_alert(h, url_)
 			, num_peers(np)
 		{ TORRENT_ASSERT(!url.empty()); }
 
@@ -358,7 +359,7 @@ namespace libtorrent
 		virtual std::string message() const
 		{
 			std::stringstream ret;
-			ret << torrent_alert::message() << " received DHT peers: "
+			ret << tracker_alert::message() << " received DHT peers: "
 				<< num_peers;
 			return ret.str();
 		}
@@ -367,8 +368,8 @@ namespace libtorrent
 	struct TORRENT_EXPORT tracker_announce_alert: tracker_alert
 	{
 		tracker_announce_alert(torrent_handle const& h
-			, std::string const& url, int event_)
-			: tracker_alert(h, url)
+			, std::string const& url_, int event_)
+			: tracker_alert(h, url_)
 			, event(event_)
 		{ TORRENT_ASSERT(!url.empty()); }
 
@@ -738,6 +739,27 @@ namespace libtorrent
 		}
 	};
 
+	struct TORRENT_EXPORT storage_moved_failed_alert: torrent_alert
+	{
+		storage_moved_failed_alert(torrent_handle const& h, error_code const& ec_)
+			: torrent_alert(h)
+			, error(ec_)
+		{}
+	
+		error_code error;
+
+		virtual std::auto_ptr<alert> clone() const
+		{ return std::auto_ptr<alert>(new storage_moved_failed_alert(*this)); }
+		virtual char const* what() const { return "storage moved failed"; }
+		const static int static_category = alert::storage_notification;
+		virtual int category() const { return static_category; }
+		virtual std::string message() const
+		{
+			return torrent_alert::message() + " storage move failed: "
+				+ error.message();
+		}
+	};
+
 	struct TORRENT_EXPORT torrent_deleted_alert: torrent_alert
 	{
 		torrent_deleted_alert(torrent_handle const& h)
