@@ -26,6 +26,7 @@
 #include "asio/io_service.hpp"
 #include "asio/socket_base.hpp"
 #include "asio/detail/bind_handler.hpp"
+#include "asio/detail/handler_base_from_member.hpp"
 #include "asio/detail/noncopyable.hpp"
 #include "asio/detail/service_base.hpp"
 #include "asio/detail/socket_holder.hpp"
@@ -83,6 +84,9 @@ public:
 
     // The protocol associated with the socket.
     protocol_type protocol_;
+
+    // Per-descriptor data used by the reactor.
+    typename Reactor::per_descriptor_data reactor_data_;
   };
 
   // The maximum number of buffers to support in a single operation.
@@ -113,7 +117,7 @@ public:
   {
     if (impl.socket_ != invalid_socket)
     {
-      reactor_.close_descriptor(impl.socket_);
+      reactor_.close_descriptor(impl.socket_, impl.reactor_data_);
 
       if (impl.flags_ & implementation_type::internal_non_blocking)
       {
@@ -155,7 +159,7 @@ public:
     if (sock.get() == invalid_socket)
       return ec;
 
-    if (int err = reactor_.register_descriptor(sock.get()))
+    if (int err = reactor_.register_descriptor(sock.get(), impl.reactor_data_))
     {
       ec = asio::error_code(err,
           asio::error::get_system_category());
@@ -180,7 +184,8 @@ public:
       return ec;
     }
 
-    if (int err = reactor_.register_descriptor(native_socket))
+    if (int err = reactor_.register_descriptor(
+          native_socket, impl.reactor_data_))
     {
       ec = asio::error_code(err,
           asio::error::get_system_category());
@@ -206,7 +211,7 @@ public:
   {
     if (is_open(impl))
     {
-      reactor_.close_descriptor(impl.socket_);
+      reactor_.close_descriptor(impl.socket_, impl.reactor_data_);
 
       if (impl.flags_ & implementation_type::internal_non_blocking)
       {
@@ -242,7 +247,7 @@ public:
       return ec;
     }
 
-    reactor_.cancel_ops(impl.socket_);
+    reactor_.cancel_ops(impl.socket_, impl.reactor_data_);
     ec = asio::error_code();
     return ec;
   }
@@ -558,28 +563,46 @@ public:
     }
   }
 
+  // Wait until data can be sent without blocking.
+  size_t send(implementation_type& impl, const null_buffers&,
+      socket_base::message_flags, asio::error_code& ec)
+  {
+    if (!is_open(impl))
+    {
+      ec = asio::error::bad_descriptor;
+      return 0;
+    }
+
+    // Wait for socket to become ready.
+    socket_ops::poll_write(impl.socket_, ec);
+
+    return 0;
+  }
+
   template <typename ConstBufferSequence, typename Handler>
-  class send_handler
+  class send_operation :
+    public handler_base_from_member<Handler>
   {
   public:
-    send_handler(socket_type socket, asio::io_service& io_service,
+    send_operation(socket_type socket, asio::io_service& io_service,
         const ConstBufferSequence& buffers, socket_base::message_flags flags,
         Handler handler)
-      : socket_(socket),
+      : handler_base_from_member<Handler>(handler),
+        socket_(socket),
         io_service_(io_service),
         work_(io_service),
         buffers_(buffers),
-        flags_(flags),
-        handler_(handler)
+        flags_(flags)
     {
     }
 
-    bool operator()(const asio::error_code& result)
+    bool perform(asio::error_code& ec,
+        std::size_t& bytes_transferred)
     {
       // Check whether the operation was successful.
-      if (result)
+      if (ec)
       {
-        io_service_.post(bind_handler(handler_, result, 0));
+        bytes_transferred = 0;
         return true;
       }
 
@@ -597,7 +620,6 @@ public:
       }
 
       // Send the data.
-      asio::error_code ec;
       int bytes = socket_ops::send(socket_, bufs, i, flags_, ec);
 
       // Check if we need to run the operation again.
@@ -605,17 +627,22 @@ public:
           || ec == asio::error::try_again)
         return false;
 
-      io_service_.post(bind_handler(handler_, ec, bytes < 0 ? 0 : bytes));
+      bytes_transferred = (bytes < 0 ? 0 : bytes);
       return true;
     }
 
+    void complete(const asio::error_code& ec,
+        std::size_t bytes_transferred)
+    {
+      io_service_.post(bind_handler(this->handler_, ec, bytes_transferred));
+    }
+
   private:
     socket_type socket_;
     asio::io_service& io_service_;
     asio::io_service::work work_;
     ConstBufferSequence buffers_;
     socket_base::message_flags flags_;
-    Handler handler_;
   };
 
   // Start an asynchronous send. The data being sent must be valid for the
@@ -666,12 +693,59 @@ public:
         impl.flags_ |= implementation_type::internal_non_blocking;
       }
 
-      reactor_.start_write_op(impl.socket_,
-          send_handler<ConstBufferSequence, Handler>(
+      reactor_.start_write_op(impl.socket_, impl.reactor_data_,
+          send_operation<ConstBufferSequence, Handler>(
             impl.socket_, this->get_io_service(), buffers, flags, handler));
     }
   }
 
+  template <typename Handler>
+  class null_buffers_operation :
+    public handler_base_from_member<Handler>
+  {
+  public:
+    null_buffers_operation(asio::io_service& io_service, Handler handler)
+      : handler_base_from_member<Handler>(handler),
+        work_(io_service)
+    {
+    }
+
+    bool perform(asio::error_code&,
+        std::size_t& bytes_transferred)
+    {
+      bytes_transferred = 0;
+      return true;
+    }
+
+    void complete(const asio::error_code& ec,
+        std::size_t bytes_transferred)
+    {
+      work_.get_io_service().post(bind_handler(
+            this->handler_, ec, bytes_transferred));
+    }
+
+  private:
+    asio::io_service::work work_;
+  };
+
+  // Start an asynchronous wait until data can be sent without blocking.
+  template <typename Handler>
+  void async_send(implementation_type& impl, const null_buffers&,
+      socket_base::message_flags, Handler handler)
+  {
+    if (!is_open(impl))
+    {
+      this->get_io_service().post(bind_handler(handler,
+            asio::error::bad_descriptor, 0));
+    }
+    else
+    {
+      reactor_.start_write_op(impl.socket_, impl.reactor_data_,
+          null_buffers_operation<Handler>(this->get_io_service(), handler),
+          false);
+    }
+  }
+
   // Send a datagram to the specified endpoint. Returns the number of bytes
   // sent.
   template <typename ConstBufferSequence>
@@ -733,29 +807,48 @@ public:
     }
   }
 
+  // Wait until data can be sent without blocking.
+  size_t send_to(implementation_type& impl, const null_buffers&,
+      socket_base::message_flags, const endpoint_type&,
+      asio::error_code& ec)
+  {
+    if (!is_open(impl))
+    {
+      ec = asio::error::bad_descriptor;
+      return 0;
+    }
+
+    // Wait for socket to become ready.
+    socket_ops::poll_write(impl.socket_, ec);
+
+    return 0;
+  }
+
   template <typename ConstBufferSequence, typename Handler>
-  class send_to_handler
+  class send_to_operation :
+    public handler_base_from_member<Handler>
   {
   public:
-    send_to_handler(socket_type socket, asio::io_service& io_service,
+    send_to_operation(socket_type socket, asio::io_service& io_service,
         const ConstBufferSequence& buffers, const endpoint_type& endpoint,
         socket_base::message_flags flags, Handler handler)
-      : socket_(socket),
+      : handler_base_from_member<Handler>(handler),
+        socket_(socket),
         io_service_(io_service),
         work_(io_service),
         buffers_(buffers),
         destination_(endpoint),
-        flags_(flags),
-        handler_(handler)
+        flags_(flags)
     {
     }
 
-    bool operator()(const asio::error_code& result)
+    bool perform(asio::error_code& ec,
+        std::size_t& bytes_transferred)
     {
       // Check whether the operation was successful.
-      if (result)
+      if (ec)
       {
-        io_service_.post(bind_handler(handler_, result, 0));
+        bytes_transferred = 0;
         return true;
       }
 
@@ -773,7 +866,6 @@ public:
       }
 
       // Send the data.
-      asio::error_code ec;
       int bytes = socket_ops::sendto(socket_, bufs, i, flags_,
           destination_.data(), destination_.size(), ec);
 
@@ -782,10 +874,16 @@ public:
           || ec == asio::error::try_again)
         return false;
 
-      io_service_.post(bind_handler(handler_, ec, bytes < 0 ? 0 : bytes));
+      bytes_transferred = (bytes < 0 ? 0 : bytes);
       return true;
     }
 
+    void complete(const asio::error_code& ec,
+        std::size_t bytes_transferred)
+    {
+      io_service_.post(bind_handler(this->handler_, ec, bytes_transferred));
+    }
+
   private:
     socket_type socket_;
     asio::io_service& io_service_;
@@ -793,7 +891,6 @@ public:
     ConstBufferSequence buffers_;
     endpoint_type destination_;
     socket_base::message_flags flags_;
-    Handler handler_;
   };
 
   // Start an asynchronous send. The data being sent must be valid for the
@@ -824,13 +921,31 @@ public:
         impl.flags_ |= implementation_type::internal_non_blocking;
       }
 
-      reactor_.start_write_op(impl.socket_,
-          send_to_handler<ConstBufferSequence, Handler>(
+      reactor_.start_write_op(impl.socket_, impl.reactor_data_,
+          send_to_operation<ConstBufferSequence, Handler>(
             impl.socket_, this->get_io_service(), buffers,
             destination, flags, handler));
     }
   }
 
+  // Start an asynchronous wait until data can be sent without blocking.
+  template <typename Handler>
+  void async_send_to(implementation_type& impl, const null_buffers&,
+      socket_base::message_flags, const endpoint_type&, Handler handler)
+  {
+    if (!is_open(impl))
+    {
+      this->get_io_service().post(bind_handler(handler,
+            asio::error::bad_descriptor, 0));
+    }
+    else
+    {
+      reactor_.start_write_op(impl.socket_, impl.reactor_data_,
+          null_buffers_operation<Handler>(this->get_io_service(), handler),
+          false);
+    }
+  }
+
   // Receive some data from the peer. Returns the number of bytes received.
   template <typename MutableBufferSequence>
   size_t receive(implementation_type& impl,
@@ -888,7 +1003,7 @@ public:
         return bytes_recvd;
 
       // Check for EOF.
-      if (bytes_recvd == 0)
+      if (bytes_recvd == 0 && impl.protocol_.type() == SOCK_STREAM)
       {
         ec = asio::error::eof;
         return 0;
@@ -906,28 +1021,48 @@ public:
     }
   }
 
+  // Wait until data can be received without blocking.
+  size_t receive(implementation_type& impl, const null_buffers&,
+      socket_base::message_flags, asio::error_code& ec)
+  {
+    if (!is_open(impl))
+    {
+      ec = asio::error::bad_descriptor;
+      return 0;
+    }
+
+    // Wait for socket to become ready.
+    socket_ops::poll_read(impl.socket_, ec);
+
+    return 0;
+  }
+
   template <typename MutableBufferSequence, typename Handler>
-  class receive_handler
+  class receive_operation :
+    public handler_base_from_member<Handler>
   {
   public:
-    receive_handler(socket_type socket, asio::io_service& io_service,
-        const MutableBufferSequence& buffers, socket_base::message_flags flags,
-        Handler handler)
-      : socket_(socket),
+    receive_operation(socket_type socket, int protocol_type,
+        asio::io_service& io_service,
+        const MutableBufferSequence& buffers,
+        socket_base::message_flags flags, Handler handler)
+      : handler_base_from_member<Handler>(handler),
+        socket_(socket),
+        protocol_type_(protocol_type),
         io_service_(io_service),
         work_(io_service),
         buffers_(buffers),
-        flags_(flags),
-        handler_(handler)
+        flags_(flags)
     {
     }
 
-    bool operator()(const asio::error_code& result)
+    bool perform(asio::error_code& ec,
+        std::size_t& bytes_transferred)
     {
       // Check whether the operation was successful.
-      if (result)
+      if (ec)
       {
-        io_service_.post(bind_handler(handler_, result, 0));
+        bytes_transferred = 0;
         return true;
       }
 
@@ -945,9 +1080,8 @@ public:
       }
 
       // Receive some data.
-      asio::error_code ec;
       int bytes = socket_ops::recv(socket_, bufs, i, flags_, ec);
-      if (bytes == 0)
+      if (bytes == 0 && protocol_type_ == SOCK_STREAM)
         ec = asio::error::eof;
 
       // Check if we need to run the operation again.
@@ -955,17 +1089,23 @@ public:
           || ec == asio::error::try_again)
         return false;
 
-      io_service_.post(bind_handler(handler_, ec, bytes < 0 ? 0 : bytes));
+      bytes_transferred = (bytes < 0 ? 0 : bytes);
       return true;
     }
 
+    void complete(const asio::error_code& ec,
+        std::size_t bytes_transferred)
+    {
+      io_service_.post(bind_handler(this->handler_, ec, bytes_transferred));
+    }
+
   private:
     socket_type socket_;
+    int protocol_type_;
     asio::io_service& io_service_;
     asio::io_service::work work_;
     MutableBufferSequence buffers_;
     socket_base::message_flags flags_;
-    Handler handler_;
   };
 
   // Start an asynchronous receive. The buffer for the data being received
@@ -1019,19 +1159,44 @@ public:
 
       if (flags & socket_base::message_out_of_band)
       {
-        reactor_.start_except_op(impl.socket_,
-            receive_handler<MutableBufferSequence, Handler>(
-              impl.socket_, this->get_io_service(), buffers, flags, handler));
+        reactor_.start_except_op(impl.socket_, impl.reactor_data_,
+            receive_operation<MutableBufferSequence, Handler>(
+              impl.socket_, impl.protocol_.type(),
+              this->get_io_service(), buffers, flags, handler));
       }
       else
       {
-        reactor_.start_read_op(impl.socket_,
-            receive_handler<MutableBufferSequence, Handler>(
-              impl.socket_, this->get_io_service(), buffers, flags, handler));
+        reactor_.start_read_op(impl.socket_, impl.reactor_data_,
+            receive_operation<MutableBufferSequence, Handler>(
+              impl.socket_, impl.protocol_.type(),
+              this->get_io_service(), buffers, flags, handler));
       }
     }
   }
 
+  // Wait until data can be received without blocking.
+  template <typename Handler>
+  void async_receive(implementation_type& impl, const null_buffers&,
+      socket_base::message_flags flags, Handler handler)
+  {
+    if (!is_open(impl))
+    {
+      this->get_io_service().post(bind_handler(handler,
+            asio::error::bad_descriptor, 0));
+    }
+    else if (flags & socket_base::message_out_of_band)
+    {
+      reactor_.start_except_op(impl.socket_, impl.reactor_data_,
+          null_buffers_operation<Handler>(this->get_io_service(), handler));
+    }
+    else
+    {
+      reactor_.start_read_op(impl.socket_, impl.reactor_data_,
+          null_buffers_operation<Handler>(this->get_io_service(), handler),
+          false);
+    }
+  }
+
   // Receive a datagram with the endpoint of the sender. Returns the number of
   // bytes received.
   template <typename MutableBufferSequence>
@@ -1087,7 +1252,7 @@ public:
       }
 
       // Check for EOF.
-      if (bytes_recvd == 0)
+      if (bytes_recvd == 0 && impl.protocol_.type() == SOCK_STREAM)
       {
         ec = asio::error::eof;
         return 0;
@@ -1105,30 +1270,53 @@ public:
     }
   }
 
+  // Wait until data can be received without blocking.
+  size_t receive_from(implementation_type& impl, const null_buffers&,
+      endpoint_type& sender_endpoint, socket_base::message_flags,
+      asio::error_code& ec)
+  {
+    if (!is_open(impl))
+    {
+      ec = asio::error::bad_descriptor;
+      return 0;
+    }
+
+    // Wait for socket to become ready.
+    socket_ops::poll_read(impl.socket_, ec);
+
+    // Reset endpoint since it can be given no sensible value at this time.
+    sender_endpoint = endpoint_type();
+
+    return 0;
+  }
+
   template <typename MutableBufferSequence, typename Handler>
-  class receive_from_handler
+  class receive_from_operation :
+    public handler_base_from_member<Handler>
   {
   public:
-    receive_from_handler(socket_type socket,
+    receive_from_operation(socket_type socket, int protocol_type,
         asio::io_service& io_service,
         const MutableBufferSequence& buffers, endpoint_type& endpoint,
         socket_base::message_flags flags, Handler handler)
-      : socket_(socket),
+      : handler_base_from_member<Handler>(handler),
+        socket_(socket),
+        protocol_type_(protocol_type),
         io_service_(io_service),
         work_(io_service),
         buffers_(buffers),
         sender_endpoint_(endpoint),
-        flags_(flags),
-        handler_(handler)
+        flags_(flags)
     {
     }
 
-    bool operator()(const asio::error_code& result)
+    bool perform(asio::error_code& ec,
+        std::size_t& bytes_transferred)
     {
       // Check whether the operation was successful.
-      if (result)
+      if (ec)
       {
-        io_service_.post(bind_handler(handler_, result, 0));
+        bytes_transferred = 0;
         return true;
       }
 
@@ -1147,10 +1335,9 @@ public:
 
       // Receive some data.
       std::size_t addr_len = sender_endpoint_.capacity();
-      asio::error_code ec;
       int bytes = socket_ops::recvfrom(socket_, bufs, i, flags_,
           sender_endpoint_.data(), &addr_len, ec);
-      if (bytes == 0)
+      if (bytes == 0 && protocol_type_ == SOCK_STREAM)
         ec = asio::error::eof;
 
       // Check if we need to run the operation again.
@@ -1159,18 +1346,24 @@ public:
         return false;
 
       sender_endpoint_.resize(addr_len);
-      io_service_.post(bind_handler(handler_, ec, bytes < 0 ? 0 : bytes));
+      bytes_transferred = (bytes < 0 ? 0 : bytes);
       return true;
     }
 
+    void complete(const asio::error_code& ec,
+        std::size_t bytes_transferred)
+    {
+      io_service_.post(bind_handler(this->handler_, ec, bytes_transferred));
+    }
+
   private:
     socket_type socket_;
+    int protocol_type_;
     asio::io_service& io_service_;
     asio::io_service::work work_;
     MutableBufferSequence buffers_;
     endpoint_type& sender_endpoint_;
     socket_base::message_flags flags_;
-    Handler handler_;
   };
 
   // Start an asynchronous receive. The buffer for the data being received and
@@ -1201,10 +1394,40 @@ public:
         impl.flags_ |= implementation_type::internal_non_blocking;
       }
 
-      reactor_.start_read_op(impl.socket_,
-          receive_from_handler<MutableBufferSequence, Handler>(
-            impl.socket_, this->get_io_service(), buffers,
-            sender_endpoint, flags, handler));
+      reactor_.start_read_op(impl.socket_, impl.reactor_data_,
+          receive_from_operation<MutableBufferSequence, Handler>(
+            impl.socket_, impl.protocol_.type(), this->get_io_service(),
+            buffers, sender_endpoint, flags, handler));
+    }
+  }
+
+  // Wait until data can be received without blocking.
+  template <typename Handler>
+  void async_receive_from(implementation_type& impl,
+      const null_buffers&, endpoint_type& sender_endpoint,
+      socket_base::message_flags flags, Handler handler)
+  {
+    if (!is_open(impl))
+    {
+      this->get_io_service().post(bind_handler(handler,
+            asio::error::bad_descriptor, 0));
+    }
+    else
+    {
+      // Reset endpoint since it can be given no sensible value at this time.
+      sender_endpoint = endpoint_type();
+
+      if (flags & socket_base::message_out_of_band)
+      {
+        reactor_.start_except_op(impl.socket_, impl.reactor_data_,
+            null_buffers_operation<Handler>(this->get_io_service(), handler));
+      }
+      else
+      {
+        reactor_.start_read_op(impl.socket_, impl.reactor_data_,
+            null_buffers_operation<Handler>(this->get_io_service(), handler),
+            false);
+      }
     }
   }
 
@@ -1299,35 +1522,32 @@ public:
   }
 
   template <typename Socket, typename Handler>
-  class accept_handler
+  class accept_operation :
+    public handler_base_from_member<Handler>
   {
   public:
-    accept_handler(socket_type socket, asio::io_service& io_service,
+    accept_operation(socket_type socket, asio::io_service& io_service,
         Socket& peer, const protocol_type& protocol,
         endpoint_type* peer_endpoint, bool enable_connection_aborted,
         Handler handler)
-      : socket_(socket),
+      : handler_base_from_member<Handler>(handler),
+        socket_(socket),
         io_service_(io_service),
         work_(io_service),
         peer_(peer),
         protocol_(protocol),
         peer_endpoint_(peer_endpoint),
-        enable_connection_aborted_(enable_connection_aborted),
-        handler_(handler)
+        enable_connection_aborted_(enable_connection_aborted)
     {
     }
 
-    bool operator()(const asio::error_code& result)
+    bool perform(asio::error_code& ec, std::size_t&)
     {
       // Check whether the operation was successful.
-      if (result)
-      {
-        io_service_.post(bind_handler(handler_, result));
+      if (ec)
         return true;
-      }
 
       // Accept the waiting connection.
-      asio::error_code ec;
       socket_holder new_socket;
       std::size_t addr_len = 0;
       if (peer_endpoint_)
@@ -1363,10 +1583,14 @@ public:
           new_socket.release();
       }
 
-      io_service_.post(bind_handler(handler_, ec));
       return true;
     }
 
+    void complete(const asio::error_code& ec, std::size_t)
+    {
+      io_service_.post(bind_handler(this->handler_, ec));
+    }
+
   private:
     socket_type socket_;
     asio::io_service& io_service_;
@@ -1375,7 +1599,6 @@ public:
     protocol_type protocol_;
     endpoint_type* peer_endpoint_;
     bool enable_connection_aborted_;
-    Handler handler_;
   };
 
   // Start an asynchronous accept. The peer and peer_endpoint objects
@@ -1409,8 +1632,8 @@ public:
         impl.flags_ |= implementation_type::internal_non_blocking;
       }
 
-      reactor_.start_read_op(impl.socket_,
-          accept_handler<Socket, Handler>(
+      reactor_.start_read_op(impl.socket_, impl.reactor_data_,
+          accept_operation<Socket, Handler>(
             impl.socket_, this->get_io_service(),
             peer, impl.protocol_, peer_endpoint,
             (impl.flags_ & implementation_type::enable_connection_aborted) != 0,
@@ -1444,70 +1667,52 @@ public:
   }
 
   template <typename Handler>
-  class connect_handler
+  class connect_operation :
+    public handler_base_from_member<Handler>
   {
   public:
-    connect_handler(socket_type socket, boost::shared_ptr<bool> completed,
-        asio::io_service& io_service, Reactor& reactor, Handler handler)
-      : socket_(socket),
-        completed_(completed),
+    connect_operation(socket_type socket,
+        asio::io_service& io_service, Handler handler)
+      : handler_base_from_member<Handler>(handler),
+        socket_(socket),
         io_service_(io_service),
-        work_(io_service),
-        reactor_(reactor),
-        handler_(handler)
+        work_(io_service)
     {
     }
 
-    bool operator()(const asio::error_code& result)
+    bool perform(asio::error_code& ec, std::size_t&)
     {
-      // Check whether a handler has already been called for the connection.
-      // If it has, then we don't want to do anything in this handler.
-      if (*completed_)
-        return true;
-
-      // Cancel the other reactor operation for the connection.
-      *completed_ = true;
-      reactor_.enqueue_cancel_ops_unlocked(socket_);
-
       // Check whether the operation was successful.
-      if (result)
-      {
-        io_service_.post(bind_handler(handler_, result));
+      if (ec)
         return true;
-      }
 
       // Get the error code from the connect operation.
       int connect_error = 0;
       size_t connect_error_len = sizeof(connect_error);
-      asio::error_code ec;
       if (socket_ops::getsockopt(socket_, SOL_SOCKET, SO_ERROR,
             &connect_error, &connect_error_len, ec) == socket_error_retval)
-      {
-        io_service_.post(bind_handler(handler_, ec));
         return true;
-      }
 
-      // If connection failed then post the handler with the error code.
+      // The connection failed so the handler will be posted with an error code.
       if (connect_error)
       {
         ec = asio::error_code(connect_error,
             asio::error::get_system_category());
-        io_service_.post(bind_handler(handler_, ec));
         return true;
       }
 
-      // Post the result of the successful connection operation.
-      io_service_.post(bind_handler(handler_, ec));
       return true;
     }
 
+    void complete(const asio::error_code& ec, std::size_t)
+    {
+      io_service_.post(bind_handler(this->handler_, ec));
+    }
+
   private:
     socket_type socket_;
-    boost::shared_ptr<bool> completed_;
     asio::io_service& io_service_;
     asio::io_service::work work_;
-    Reactor& reactor_;
-    Handler handler_;
   };
 
   // Start an asynchronous connect.
@@ -1551,10 +1756,9 @@ public:
     {
       // The connection is happening in the background, and we need to wait
       // until the socket becomes writeable.
-      boost::shared_ptr<bool> completed(new bool(false));
-      reactor_.start_write_and_except_ops(impl.socket_,
-          connect_handler<Handler>(impl.socket_, completed,
-            this->get_io_service(), reactor_, handler));
+      reactor_.start_connect_op(impl.socket_, impl.reactor_data_,
+          connect_operation<Handler>(impl.socket_,
+            this->get_io_service(), handler));
     }
     else
     {
