@@ -85,6 +85,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/assert.hpp"
 #include "libtorrent/udp_socket.hpp"
 
+#if TORRENT_COMPLETE_TYPES_REQUIRED
+#include "libtorrent/peer_connection.hpp"
+#endif
+
 namespace libtorrent
 {
 
@@ -184,6 +188,8 @@ namespace libtorrent
 			void stop_dht();
 
 			entry dht_state() const;
+			void on_dht_router_name_lookup(error_code const& e
+				, tcp::resolver::iterator host);
 #endif
 
 #ifndef TORRENT_DISABLE_ENCRYPTION
@@ -356,6 +362,8 @@ namespace libtorrent
 			// them
 			mutable io_service m_io_service;
 
+			tcp::resolver m_host_resolver;
+
 			// handles disk io requests asynchronously
 			// peers have pointers into the disk buffer
 			// pool, and must be destructed before this
@@ -541,7 +549,7 @@ namespace libtorrent
 
 			// these are used when starting the DHT
 			// (and bootstrapping it), and then erased
-			std::list<std::pair<std::string, int> > m_dht_router_nodes;
+			std::list<udp::endpoint> m_dht_router_nodes;
 
 			void on_receive_udp(error_code const& e
 				, udp::endpoint const& ep, char const* buf, int len);
