@@ -69,7 +69,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #pragma warning(push, 1)
 #endif
 
-#include <boost/lexical_cast.hpp>
 #include <boost/static_assert.hpp>
 
 #ifdef _MSC_VER
@@ -80,17 +79,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/config.hpp"
 
 #include "libtorrent/assert.hpp"
-
-#if defined(_MSC_VER)
-namespace std
-{
-	using ::isdigit;
-	using ::atoi;
-};
-
-#define for if (false) {}  else for
-#endif
-
+#include "libtorrent/escape_string.hpp"
 
 namespace libtorrent
 {
@@ -238,7 +227,7 @@ namespace libtorrent
 			if (in == end)
 			{
 				err = true;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				ret.m_type_queried = false;
 #endif
 				return;
@@ -256,10 +245,20 @@ namespace libtorrent
 				TORRENT_ASSERT(*in == 'e');
 				++in; // 'e' 
 				ret = entry(entry::int_t);
-				ret.integer() = boost::lexical_cast<entry::integer_type>(val);
-#ifndef NDEBUG
+				char* end_pointer;
+#if defined TORRENT_WINDOWS && !defined TORRENT_MINGW
+				ret.integer() = _strtoi64(val.c_str(), &end_pointer, 10);
+#else
+				ret.integer() = strtoll(val.c_str(), &end_pointer, 10);
+#endif
+#ifdef TORRENT_DEBUG
 				ret.m_type_queried = false;
 #endif
+				if (end_pointer == val.c_str())
+				{
+					err = true;
+					return;
+				}
 				} break;
 
 			// ----------------------------------------------
@@ -275,7 +274,7 @@ namespace libtorrent
 					bdecode_recursive(in, end, e, err, depth + 1);
 					if (err)
 					{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
@@ -283,13 +282,13 @@ namespace libtorrent
 					if (in == end)
 					{
 						err = true;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
 					}
 				}
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				ret.m_type_queried = false;
 #endif
 				TORRENT_ASSERT(*in == 'e');
@@ -308,7 +307,7 @@ namespace libtorrent
 					bdecode_recursive(in, end, key, err, depth + 1);
 					if (err || key.type() != entry::string_t)
 					{	
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
@@ -317,7 +316,7 @@ namespace libtorrent
 					bdecode_recursive(in, end, e, err, depth + 1);
 					if (err)
 					{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
@@ -325,13 +324,13 @@ namespace libtorrent
 					if (in == end)
 					{
 						err = true;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
 					}
 				}
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				ret.m_type_queried = false;
 #endif
 				TORRENT_ASSERT(*in == 'e');
@@ -341,12 +340,12 @@ namespace libtorrent
 			// ----------------------------------------------
 			// string
 			default:
-				if (isdigit((unsigned char)*in))
+				if (is_digit((unsigned char)*in))
 				{
 					std::string len_s = read_until(in, end, ':', err);
 					if (err)
 					{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
@@ -358,7 +357,7 @@ namespace libtorrent
 					read_string(in, end, len, ret.string(), err);
 					if (err)
 					{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 						ret.m_type_queried = false;
 #endif
 						return;
@@ -367,12 +366,12 @@ namespace libtorrent
 				else
 				{
 					err = true;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 					ret.m_type_queried = false;
 #endif
 					return;
 				}
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				ret.m_type_queried = false;
 #endif
 			}
@@ -391,7 +390,9 @@ namespace libtorrent
 		entry e;
 		bool err = false;
 		detail::bdecode_recursive(start, end, e, err, 0);
+#ifdef TORRENT_DEBUG
 		TORRENT_ASSERT(e.m_type_queried == false);
+#endif
 		if (err) return entry();
 		return e;
 	}
@@ -411,3 +412,4 @@ namespace libtorrent
 }
 
 #endif // TORRENT_BENCODE_HPP_INCLUDED
+
