@@ -71,14 +71,15 @@ namespace libtorrent { namespace dht
 	{
 		friend void intrusive_ptr_add_ref(dht_tracker const*);
 		friend void intrusive_ptr_release(dht_tracker const*);
-		dht_tracker(udp_socket& sock, dht_settings const& settings);
+		dht_tracker(udp_socket& sock, dht_settings const& settings
+			, entry const* state);
 
 		void start(entry const& bootstrap);
 		void stop();
 
 		void add_node(udp::endpoint node);
 		void add_node(std::pair<std::string, int> const& node);
-		void add_router_node(std::pair<std::string, int> const& node);
+		void add_router_node(udp::endpoint const& node);
 
 		entry state() const;
 
