@@ -37,6 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/intrusive_ptr_base.hpp"
 
 #include <boost/function.hpp>
+#include <boost/thread/mutex.hpp>
 
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 #include <fstream>
@@ -45,8 +46,8 @@ POSSIBILITY OF SUCH DAMAGE.
 namespace libtorrent
 {
 
-// int: external tcp port
-// int: external udp port
+// int: port mapping index
+// int: external port
 // std::string: error message
 typedef boost::function<void(int, int, std::string const&)> portmap_callback_t;
 
@@ -59,34 +60,39 @@ public:
 
 	// maps the ports, if a port is set to 0
 	// it will not be mapped
-	void set_mappings(int tcp, int udp);
+	enum protocol_type { none = 0, udp = 1, tcp = 2 };
+	int add_mapping(protocol_type p, int external_port, int local_port);
+	void delete_mapping(int mapping_index);
 
 	void close();
 
 private:
 	
-	void update_mapping(int i, int port);
+	void update_mapping(int i);
 	void send_map_request(int i);
-	void resend_request(int i, asio::error_code const& e);
-	void on_reply(asio::error_code const& e
+	void resend_request(int i, error_code const& e);
+	void on_reply(error_code const& e
 		, std::size_t bytes_transferred);
 	void try_next_mapping(int i);
 	void update_expiration_timer();
-	void refresh_mapping(int i);
-	void mapping_expired(asio::error_code const& e, int i);
+	void mapping_expired(error_code const& e, int i);
+	void close_impl();
 
-	struct mapping
+	void disable(char const* message);
+
+	struct mapping_t
 	{
-		mapping()
-			: need_update(false)
+		enum action_t { action_none, action_add, action_delete };
+		mapping_t()
+			: action(action_none)
 			, local_port(0)
 			, external_port(0)
-			, protocol(1)
+			, protocol(none)
 		{}
 
 		// indicates that the mapping has changed
 		// and needs an update
-		bool need_update;
+		int action;
 
 		// the time the port mapping will expire
 		ptime expires;
@@ -100,14 +106,12 @@ private:
 		// should announce to others
 		int external_port;
 
-		// 1 = udp, 2 = tcp
 		int protocol;
 	};
 
 	portmap_callback_t m_callback;
 
-	// 0 is tcp and 1 is udp
-	mapping m_mappings[2];
+	std::vector<mapping_t> m_mappings;
 	
 	// the endpoint to the nat router
 	udp::endpoint m_nat_endpoint;
@@ -136,9 +140,17 @@ private:
 
 	// timer used to refresh mappings
 	deadline_timer m_refresh_timer;
+
+	// the mapping index that will expire next
+	int m_next_refresh;
 	
 	bool m_disabled;
 
+	bool m_abort;
+
+	typedef boost::mutex mutex_t;
+	mutex_t m_mutex;
+
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	std::ofstream m_log;
 #endif
