@@ -99,6 +99,9 @@ namespace libtorrent
 		// the number of times the request
 		// has been skipped by out of order blocks
 		piece_block block;
+
+		bool operator==(pending_block const& b)
+		{ return b.skipped == skipped && b.block == block; }
 	};
 
 	struct has_block
@@ -150,7 +153,10 @@ namespace libtorrent
 		virtual ~peer_connection();
 
 		void set_peer_info(policy::peer* pi)
-		{ m_peer_info = pi; }
+		{
+			TORRENT_ASSERT(m_peer_info == 0 || pi == 0 );
+			m_peer_info = pi;
+		}
 
 		policy::peer* peer_info_struct() const
 		{ return m_peer_info; }
@@ -585,6 +591,10 @@ namespace libtorrent
 		void on_disk_write_complete(int ret, disk_io_job const& j
 			, peer_request r, boost::shared_ptr<torrent> t);
 
+		// keep the io_service running as long as we
+		// have peer connections
+		io_service::work m_work;
+
 		// the time when we last got a part of a
 		// piece packet from this peer
 		ptime m_last_piece;
