@@ -52,13 +52,14 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/session_settings.hpp"
 #include "libtorrent/config.hpp"
 #include "libtorrent/assert.hpp"
+#include "libtorrent/time.hpp"
 
 namespace libtorrent
 {
 
 	class torrent;
 	class peer_connection;
-	class bitfield;
+	struct bitfield;
 
 	struct TORRENT_EXPORT piece_block
 	{
@@ -134,8 +135,9 @@ namespace libtorrent
 
 		struct downloading_piece
 		{
-			downloading_piece(): finished(0), writing(0), requested(0) {}
+			downloading_piece(): last_request(min_time()), finished(0), writing(0), requested(0) {}
 			piece_state_t state;
+			ptime last_request;
 
 			// the index of the piece
 			int index;
@@ -184,7 +186,7 @@ namespace libtorrent
 		int reverse_cursor() const { return m_reverse_cursor; }
 
 		// sets all pieces to dont-have
-		void init(int blocks_per_piece, int total_num_blocks);
+		void init(int blocks_per_piece, int blocks_in_last_piece, int total_num_pieces);
 		int num_pieces() const { return int(m_piece_map.size()); }
 
 		bool have_piece(int index) const
@@ -272,6 +274,7 @@ namespace libtorrent
 		void mark_as_finished(piece_block block, void* peer);
 		void write_failed(piece_block block);
 		int num_peers(piece_block block) const;
+		ptime last_request(int piece) const;
 
 		// returns information about the given piece
 		void piece_info(int index, piece_picker::downloading_piece& st) const;
@@ -308,14 +311,14 @@ namespace libtorrent
 
 		int num_have() const { return m_num_have; }
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		// used in debug mode
 		void verify_priority(int start, int end, int prio) const;
 		void check_invariant(const torrent* t = 0) const;
 		void verify_pick(std::vector<piece_block> const& picked
 			, bitfield const& bits) const;
 #endif
-#if defined TORRENT_PICKER_LOG || !defined NDEBUG
+#if defined TORRENT_PICKER_LOG || defined TORRENT_DEBUG
 		void print_pieces() const;
 #endif
 
