@@ -96,6 +96,8 @@ namespace libtorrent
 		// the given connection was just closed
 		void connection_closed(const peer_connection& c);
 
+		void ban_peer(policy::peer* p);
+
 		// the peer has got at least one interesting piece
 		void peer_is_interesting(peer_connection& c);
 
@@ -110,7 +112,9 @@ namespace libtorrent
 
 		void ip_filter_updated();
 
-#ifndef NDEBUG
+		void set_seed(policy::peer* p, bool s);
+
+#ifdef TORRENT_DEBUG
 		bool has_connection(const peer_connection* p);
 
 		void check_invariant() const;
@@ -156,7 +160,7 @@ namespace libtorrent
 			peer_connection* connection;
 
 #ifndef TORRENT_DISABLE_GEO_IP
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 			// only used in debug mode to assert that
 			// the first entry in the AS pair keeps the same
 			boost::uint16_t inet_as_num;
@@ -246,6 +250,8 @@ namespace libtorrent
 		iterator end_peer() { return m_peers.end(); }
 		const_iterator begin_peer() const { return m_peers.begin(); }
 		const_iterator end_peer() const { return m_peers.end(); }
+		std::pair<iterator, iterator> find_peers(address const& a)
+		{ return m_peers.equal_range(a); }
 
 		bool connect_one_peer();
 
@@ -253,11 +259,7 @@ namespace libtorrent
 
 		int num_seeds() const { return m_num_seeds; }
 		int num_connect_candidates() const { return m_num_connect_candidates; }
-		void recalculate_connect_candidates()
-		{
-			if (m_num_connect_candidates == 0)
-				m_num_connect_candidates = 1;
-		}
+		void recalculate_connect_candidates();
 
 		void erase_peer(iterator i);
 
@@ -268,7 +270,7 @@ namespace libtorrent
 
 		iterator find_connect_candidate();
 
-		bool is_connect_candidate(peer const& p, bool finished);
+		bool is_connect_candidate(peer const& p, bool finished) const;
 
 		std::multimap<address, peer> m_peers;
 
@@ -292,6 +294,16 @@ namespace libtorrent
 
 		// the number of seeds in the peer list
 		int m_num_seeds;
+
+		// this was the state of the torrent the
+		// last time we recalculated the number of
+		// connect candidates. Since seeds (or upload
+		// only) peers are not connect candidates
+		// when we're finished, the set depends on
+		// this state. Every time m_torrent->is_finished()
+		// is different from this state, we need to
+		// recalculate the connect candidates.
+		bool m_finished;
 	};
 
 }
