@@ -40,6 +40,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #pragma warning(push, 1)
 #endif
 
+#include <boost/function.hpp>
 #include <boost/limits.hpp>
 #include <boost/thread.hpp>
 #include <boost/shared_ptr.hpp>
@@ -80,7 +81,7 @@ namespace libtorrent
 		storage_mode_compact
 	};
 	
-#if defined(_WIN32) && defined(UNICODE)
+#if TORRENT_USE_WPATH
 
 	TORRENT_EXPORT std::wstring safe_convert(std::string const& s);
 
@@ -123,6 +124,8 @@ namespace libtorrent
 		// false return value indicates an error
 		virtual bool initialize(bool allocate_files) = 0;
 
+		virtual bool has_any_file() = 0;
+
 		// negative return value indicates an error
 		virtual int read(char* buf, int slot, int offset, int size) = 0;
 
@@ -181,12 +184,10 @@ namespace libtorrent
 	};
 
 	typedef storage_interface* (&storage_constructor_type)(
-		file_storage const&, fs::path const&, file_pool&);
+		file_storage const&, file_storage const*, fs::path const&, file_pool&);
 
 	TORRENT_EXPORT storage_interface* default_storage_constructor(
-		file_storage const&, fs::path const&, file_pool&);
-	TORRENT_EXPORT storage_interface* mapped_storage_constructor(
-		file_storage const&, fs::path const&, file_pool&);
+		file_storage const&, file_storage const* mapped, fs::path const&, file_pool&);
 
 	struct disk_io_thread;
 
@@ -236,6 +237,8 @@ namespace libtorrent
 			boost::function<void(int, disk_io_job const&)> const& handler
 			= boost::function<void(int, disk_io_job const&)>());
 
+		void abort_disk_io();
+
 		void async_clear_read_cache(
 			boost::function<void(int, disk_io_job const&)> const& handler
 			= boost::function<void(int, disk_io_job const&)>());
@@ -296,7 +299,7 @@ namespace libtorrent
 		bool compact_allocation() const
 		{ return m_storage_mode == storage_mode_compact; }
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		std::string name() const { return m_info->name(); }
 #endif
 
@@ -331,7 +334,7 @@ namespace libtorrent
 		int move_storage_impl(fs::path const& save_path);
 
 		int allocate_slot_for_piece(int piece_index);
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		void check_invariant() const;
 #ifdef TORRENT_STORAGE_DEBUG
 		void debug_log() const;
