@@ -34,6 +34,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_UDP_SOCKET_HPP_INCLUDED
 
 #include "libtorrent/socket.hpp"
+#include "libtorrent/buffer.hpp"
 #include "libtorrent/session_settings.hpp"
 
 #include <vector>
@@ -65,10 +66,20 @@ namespace libtorrent
 		void set_proxy_settings(proxy_settings const& ps);
 		proxy_settings const& get_proxy_settings() { return m_proxy_settings; }
 
+	protected:
+
+		struct queued_packet
+		{
+			udp::endpoint ep;
+			buffer buf;
+		};
+
 	private:
 
 		callback_t m_callback;
 
+		typedef boost::mutex mutex_t;
+
 		void on_read(udp::socket* sock, error_code const& e, std::size_t bytes_transferred);
 		void on_name_lookup(error_code const& e, tcp::resolver::iterator i);
 		void on_timeout();
@@ -78,14 +89,13 @@ namespace libtorrent
 		void handshake2(error_code const& e);
 		void handshake3(error_code const& e);
 		void handshake4(error_code const& e);
-		void socks_forward_udp();
+		void socks_forward_udp(mutex_t::scoped_lock& l);
 		void connect1(error_code const& e);
 		void connect2(error_code const& e);
 
 		void wrap(udp::endpoint const& ep, char const* p, int len, error_code& ec);
 		void unwrap(error_code const& e, char const* buf, int size);
 
-		typedef boost::mutex mutex_t;
 		mutable mutex_t m_mutex;
 
 		udp::socket m_ipv4_sock;
@@ -103,10 +113,16 @@ namespace libtorrent
 		connection_queue& m_cc;
 		tcp::resolver m_resolver;
 		char m_tmp_buf[100];
+		bool m_queue_packets;
 		bool m_tunnel_packets;
 		bool m_abort;
 		udp::endpoint m_proxy_addr;
+		// while we're connecting to the proxy
+		// we have to queue the packets, we'll flush
+		// them once we're connected
+		std::list<queued_packet> m_queue;
 #ifdef TORRENT_DEBUG
+		bool m_started;
 		int m_magic;
 #endif
 	};
