@@ -97,6 +97,7 @@ namespace libtorrent
 			, tcp::endpoint const& remote
 			, std::string const& url
 			, policy::peer* peerinfo);
+		void start();
 
 		~web_peer_connection();
 
@@ -107,7 +108,7 @@ namespace libtorrent
 		void on_receive(error_code const& error
 			, std::size_t bytes_transferred);
 			
-		std::string const& url() const { return m_url; }
+		std::string const& url() const { return m_original_url; }
 		
 		virtual void get_specific_peer_info(peer_info& p) const;
 		virtual bool in_handshake() const;
@@ -128,7 +129,7 @@ namespace libtorrent
 		void write_reject_request(peer_request const&) {}
 		void write_allow_fast(int) {}
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		void check_invariant() const;
 #endif
 
@@ -154,6 +155,7 @@ namespace libtorrent
 		int m_port;
 		std::string m_path;
 		std::string m_url;
+		const std::string m_original_url;
 			
 		// the first request will contain a little bit more data
 		// than subsequent ones, things that aren't critical are left
