@@ -328,9 +328,10 @@ namespace libtorrent
 	{
 		INVARIANT_CHECK;
 
+		if (!m_supports_fast) return;
+
 		TORRENT_ASSERT(m_sent_handshake && m_sent_bitfield);
 		TORRENT_ASSERT(associated_torrent().lock()->valid_metadata());
-		TORRENT_ASSERT(m_supports_fast);
 
 		char msg[] = {0,0,0,5, msg_allowed_fast, 0, 0, 0, 0};
 		char* ptr = msg + 5;
@@ -1137,12 +1138,6 @@ namespace libtorrent
 	{
 		INVARIANT_CHECK;
 
-		if (!m_supports_dht_port)
-		{
-			disconnect("got 'dht_port' message from peer that doesn't support it", 2);
-			return;
-		}
-
 		TORRENT_ASSERT(received > 0);
 		if (packet_size() != 3)
 		{
@@ -1519,7 +1514,11 @@ namespace libtorrent
 		int num_lazy_pieces = 0;
 		int lazy_piece = 0;
 
-		if (t->is_seed() && m_ses.settings().lazy_bitfields)
+		if (t->is_seed() && m_ses.settings().lazy_bitfields
+#ifndef TORRENT_DISABLE_ENCRYPTION
+			&& !m_encrypted
+#endif
+			)
 		{
 			num_lazy_pieces = (std::min)(50, num_pieces / 10);
 			if (num_lazy_pieces < 1) num_lazy_pieces = 1;
@@ -1631,7 +1630,12 @@ namespace libtorrent
 		handshake["reqq"] = m_ses.settings().max_allowed_in_request_queue;
 		boost::shared_ptr<torrent> t = associated_torrent().lock();
 		TORRENT_ASSERT(t);
-		if (t->is_finished()) handshake["upload_only"] = 1;
+		if (t->is_finished() && (!m_ses.settings().lazy_bitfields
+#ifndef TORRENT_DISABLE_ENCRYPTION
+			|| m_encrypted
+#endif
+			))
+			handshake["upload_only"] = 1;
 
 		tcp::endpoint ep = m_ses.get_ipv6_interface();
 		if (!is_any(ep.address()))
