@@ -1,4 +1,3 @@
-
 /*
 
 Copyright (c) 2007, Arvid Norberg
@@ -44,7 +43,7 @@ namespace libtorrent
 		, m_half_open_limit(0)
 		, m_abort(false)
 		, m_timer(ios)
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		, m_in_timeout_function(false)
 #endif
 	{
@@ -141,7 +140,7 @@ namespace libtorrent
 	int connection_queue::limit() const
 	{ return m_half_open_limit; }
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 
 	void connection_queue::check_invariant() const
 	{
@@ -177,6 +176,9 @@ namespace libtorrent
 
 		std::list<entry>::iterator i = std::find_if(m_queue.begin()
 			, m_queue.end(), boost::bind(&entry::connecting, _1) == false);
+
+		std::list<entry> to_connect;
+
 		while (i != m_queue.end())
 		{
 			TORRENT_ASSERT(i->connecting == false);
@@ -193,15 +195,7 @@ namespace libtorrent
 
 			INVARIANT_CHECK;
 
-			entry& ent = *i;
-			++i;
-#ifndef BOOST_NO_EXCEPTIONS
-			try {
-#endif
-				ent.on_connect(ent.ticket);
-#ifndef BOOST_NO_EXCEPTIONS
-			} catch (std::exception&) {}
-#endif
+			to_connect.push_back(*i);
 
 #ifdef TORRENT_CONNECTION_LOGGING
 			m_log << log_time() << " " << free_slots() << std::endl;
@@ -211,9 +205,22 @@ namespace libtorrent
 				&& m_half_open_limit > 0) break;
 			i = std::find_if(i, m_queue.end(), boost::bind(&entry::connecting, _1) == false);
 		}
+
+		while (!to_connect.empty())
+		{
+			entry& ent = to_connect.front();
+#ifndef BOOST_NO_EXCEPTIONS
+			try {
+#endif
+				ent.on_connect(ent.ticket);
+#ifndef BOOST_NO_EXCEPTIONS
+			} catch (std::exception&) {}
+#endif
+			to_connect.pop_front();
+		}
 	}
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 	struct function_guard
 	{
 		function_guard(bool& v): val(v) { TORRENT_ASSERT(!val); val = true; }
@@ -228,7 +235,7 @@ namespace libtorrent
 		mutex_t::scoped_lock l(m_mutex);
 
 		INVARIANT_CHECK;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		function_guard guard_(m_in_timeout_function);
 #endif
 
