@@ -34,6 +34,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <cctype>
 #include <algorithm>
+#include <stdlib.h>
 
 #include "libtorrent/config.hpp"
 #include "libtorrent/http_parser.hpp"
@@ -114,6 +115,8 @@ namespace libtorrent
 			{
 				m_method = m_protocol;
 				std::transform(m_method.begin(), m_method.end(), m_method.begin(), &to_lower);
+				// the content length is assumed to be 0 for requests
+				m_content_length = 0;
 				m_protocol.clear();
 				line >> m_path >> m_protocol;
 				m_status_code = 0;
@@ -162,7 +165,7 @@ namespace libtorrent
 
 				if (name == "content-length")
 				{
-#ifdef WIN32
+#ifdef TORRENT_WINDOWS
 					m_content_length = _atoi64(value.c_str());
 #else
 					m_content_length = atoll(value.c_str());
