@@ -37,12 +37,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/socket_type.hpp"
 #include <boost/shared_ptr.hpp>
 #include <stdexcept>
-#include <asio/io_service.hpp>
 
 namespace libtorrent
 {
 
-	bool instantiate_connection(asio::io_service& ios
+	bool instantiate_connection(io_service& ios
 		, proxy_settings const& ps, socket_type& s)
 	{
 		if (ps.type == proxy_settings::none)
@@ -53,23 +52,23 @@ namespace libtorrent
 			|| ps.type == proxy_settings::http_pw)
 		{
 			s.instantiate<http_stream>(ios);
-			s.get<http_stream>().set_proxy(ps.hostname, ps.port);
+			s.get<http_stream>()->set_proxy(ps.hostname, ps.port);
 			if (ps.type == proxy_settings::http_pw)
-				s.get<http_stream>().set_username(ps.username, ps.password);
+				s.get<http_stream>()->set_username(ps.username, ps.password);
 		}
 		else if (ps.type == proxy_settings::socks5
 			|| ps.type == proxy_settings::socks5_pw)
 		{
 			s.instantiate<socks5_stream>(ios);
-			s.get<socks5_stream>().set_proxy(ps.hostname, ps.port);
+			s.get<socks5_stream>()->set_proxy(ps.hostname, ps.port);
 			if (ps.type == proxy_settings::socks5_pw)
-				s.get<socks5_stream>().set_username(ps.username, ps.password);
+				s.get<socks5_stream>()->set_username(ps.username, ps.password);
 		}
 		else if (ps.type == proxy_settings::socks4)
 		{
 			s.instantiate<socks4_stream>(ios);
-			s.get<socks4_stream>().set_proxy(ps.hostname, ps.port);
-			s.get<socks4_stream>().set_username(ps.username);
+			s.get<socks4_stream>()->set_proxy(ps.hostname, ps.port);
+			s.get<socks4_stream>()->set_username(ps.username);
 		}
 		else
 		{
