@@ -39,7 +39,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <boost/bind.hpp>
 
 using boost::bind;
-using asio::ip::udp;
 
 namespace libtorrent { namespace dht
 {
@@ -89,7 +88,8 @@ boost::pool<>& traversal_algorithm::allocator() const
 void traversal_algorithm::traverse(node_id const& id, udp::endpoint addr)
 {
 #ifdef TORRENT_DHT_VERBOSE_LOGGING
-	TORRENT_LOG(traversal) << "node returned a list which included a node with id 0";
+	if (id.is_all_zeros())
+	TORRENT_LOG(traversal) << time_now_string() << " WARNING: node returned a list which included a node with id 0";
 #endif
 	add_entry(id, addr, 0);
 }
@@ -112,9 +112,9 @@ void traversal_algorithm::failed(node_id const& id, bool prevent_request)
 	std::vector<result>::iterator i = std::find_if(
 		m_results.begin()
 		, m_results.end()
-		, bind(
+		, boost::bind(
 			std::equal_to<node_id>()
-			, bind(&result::id, _1)
+			, boost::bind(&result::id, _1)
 			, id
 		)
 	);
@@ -160,9 +160,9 @@ void traversal_algorithm::add_requests()
 		std::vector<result>::iterator i = std::find_if(
 			m_results.begin()
 			, last_iterator()
-			, bind(
+			, boost::bind(
 				&bitwise_nand
-				, bind(&result::flags, _1)
+				, boost::bind(&result::flags, _1)
 				, (unsigned char)result::queried
 			)
 		);
