@@ -31,26 +31,24 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/lazy_entry.hpp"
+#include "libtorrent/escape_string.hpp"
 #include <iostream>
 #include <iomanip>
 #include <cstring>
 
 namespace
 {
-	enum
-	{
-		lazy_entry_grow_factor = 3,
-		lazy_entry_dict_init = 30,
-		lazy_entry_list_init = 50
-	};
+	const float lazy_entry_grow_factor = 1.5f;
+	const int lazy_entry_dict_init = 5;
+	const int lazy_entry_list_init = 5;
 }
 
 namespace libtorrent
 {
-	int fail_bdecode(lazy_entry& ret)
+	int fail_bdecode(lazy_entry& ret, int return_value = -1)
 	{
 		ret = lazy_entry();
-		return -1;
+		return return_value;
 	}
 
 	// fills in 'val' with what the string between start and the
@@ -61,8 +59,7 @@ namespace libtorrent
 	{
 		while (start < end && *start != delimiter)
 		{
-			using namespace std;
-			if (!isdigit(*start)) { return 0; }
+			if (!is_digit(*start)) { return 0; }
 			val *= 10;
 			val += *start - '0';
 			++start;
@@ -111,10 +108,11 @@ namespace libtorrent
 					start = parse_int(start, end, ':', len);
 					if (start == 0 || start + len + 3 > end || *start != ':') return fail_bdecode(ret);
 					++start;
-					if (start == end) fail_bdecode(ret);
+					if (start == end) return fail_bdecode(ret);
 					lazy_entry* ent = top->dict_append(start);
+					if (ent == 0) return fail_bdecode(ret, -2);
 					start += len;
-					if (start >= end) fail_bdecode(ret);
+					if (start >= end) return fail_bdecode(ret);
 					stack.push_back(ent);
 					t = *start;
 					++start;
@@ -129,6 +127,7 @@ namespace libtorrent
 						continue;
 					}
 					lazy_entry* ent = top->list_append();
+					if (ent == 0) return fail_bdecode(ret, -2);
 					stack.push_back(ent);
 					break;
 				}
@@ -157,8 +156,7 @@ namespace libtorrent
 				}
 				default:
 				{
-					using namespace std;
-					if (!isdigit(t)) return fail_bdecode(ret);
+					if (!is_digit(t)) return fail_bdecode(ret);
 
 					boost::int64_t len = t - '0';
 					start = parse_int(start, end, ':', len);
@@ -389,7 +387,7 @@ namespace libtorrent
 				for (int i = 0; i < e.string_length(); ++i)
 					os << std::hex << std::setfill('0') << std::setw(2)
 					<< int((unsigned char)(str[i]));
-				return os << "'";
+				return os << "'" << std::dec;
 			}
 			case lazy_entry::list_t:
 			{
@@ -399,8 +397,8 @@ namespace libtorrent
 						&& e.list_size() < 20)
 					|| (e.list_at(0)->type() == lazy_entry::string_t
 						&& (e.list_at(0)->string_length() < 10
-							|| e.list_size() < 2))
-						&& e.list_size() < 5);
+							|| e.list_size() < 2)
+						&& e.list_size() < 5));
 				if (!one_liner) os << "\n";
 				for (int i = 0; i < e.list_size(); ++i)
 				{
