@@ -37,6 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/http_tracker_connection.hpp"
 #include "libtorrent/buffer.hpp"
 #include "libtorrent/http_parser.hpp"
+#include "libtorrent/escape_string.hpp"
 
 #include <boost/bind.hpp>
 #include <boost/ref.hpp>
@@ -60,11 +61,13 @@ namespace libtorrent
 	address guess_local_address(io_service&);
 }
 
+static error_code ec;
+
 lsd::lsd(io_service& ios, address const& listen_interface
 	, peer_callback_t const& cb)
 	: m_callback(cb)
 	, m_retry_count(1)
-	, m_socket(ios, udp::endpoint(address_v4::from_string("239.192.152.143"), 6771)
+	, m_socket(ios, udp::endpoint(address_v4::from_string("239.192.152.143", ec), 6771)
 		, bind(&lsd::on_announce, self(), _1, _2, _3))
 	, m_broadcast_timer(ios)
 	, m_disabled(false)
@@ -83,7 +86,7 @@ void lsd::announce(sha1_hash const& ih, int listen_port)
 	std::stringstream btsearch;
 	btsearch << "BT-SEARCH * HTTP/1.1\r\n"
 		"Host: 239.192.152.143:6771\r\n"
-		"Port: " << listen_port << "\r\n"
+		"Port: " << to_string(listen_port).elems << "\r\n"
 		"Infohash: " << ih << "\r\n"
 		"\r\n\r\n";
 	std::string const& msg = btsearch.str();
@@ -99,11 +102,11 @@ void lsd::announce(sha1_hash const& ih, int listen_port)
 
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	m_log << time_now_string()
-		<< " ==> announce: ih: " << ih << " port: " << listen_port << std::endl;
+		<< " ==> announce: ih: " << ih << " port: " << to_string(listen_port).elems << std::endl;
 #endif
 
 	m_broadcast_timer.expires_from_now(milliseconds(250 * m_retry_count), ec);
-	m_broadcast_timer.async_wait(bind(&lsd::resend_announce, self(), _1, msg));
+	m_broadcast_timer.async_wait(boost::bind(&lsd::resend_announce, self(), _1, msg));
 }
 
 void lsd::resend_announce(error_code const& e, std::string msg)
@@ -118,7 +121,7 @@ void lsd::resend_announce(error_code const& e, std::string msg)
 		return;
 
 	m_broadcast_timer.expires_from_now(milliseconds(250 * m_retry_count), ec);
-	m_broadcast_timer.async_wait(bind(&lsd::resend_announce, self(), _1, msg));
+	m_broadcast_timer.async_wait(boost::bind(&lsd::resend_announce, self(), _1, msg));
 }
 
 void lsd::on_announce(udp::endpoint const& from, char* buffer
@@ -173,14 +176,14 @@ void lsd::on_announce(udp::endpoint const& from, char* buffer
 	sha1_hash ih(0);
 	std::istringstream ih_sstr(ih_str);
 	ih_sstr >> ih;
-	int port = atoi(port_str.c_str());
+	int port = std::atoi(port_str.c_str());
 
 	if (!ih.is_all_zeros() && port != 0)
 	{
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 		m_log << time_now_string()
 			<< " *** incoming local announce " << from.address()
-			<< ":" << port << " ih: " << ih << std::endl;
+			<< ":" << to_string(port).elems << " ih: " << ih << std::endl;
 #endif
 		// we got an announce, pass it on through the callback
 #ifndef BOOST_NO_EXCEPTIONS
