@@ -31,6 +31,11 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/pch.hpp"
+#include "libtorrent/config.hpp"
+
+#ifdef TORRENT_WINDOWS
+#include <winsock2.h> // for error codes
+#endif
 
 #include <boost/version.hpp>
 #include <boost/bind.hpp>
@@ -58,6 +63,7 @@ natpmp::natpmp(io_service& ios, address const& listen_interface, portmap_callbac
 	, m_refresh_timer(ios)
 	, m_next_refresh(-1)
 	, m_disabled(false)
+	, m_abort(false)
 {
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	m_log.open("natpmp.log", std::ios::in | std::ios::out | std::ios::trunc);
@@ -134,7 +140,7 @@ void natpmp::disable(char const* message)
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	m_log << time_now_string() << " NAT-PMP disabled: " << message << std::endl;
 #endif
-	close();
+	close_impl();
 }
 void natpmp::delete_mapping(int index)
 {
@@ -287,10 +293,22 @@ void natpmp::send_map_request(int i)
 
 	error_code ec;
 	m_socket.send_to(asio::buffer(buf, 12), m_nat_endpoint, 0, ec);
-	// linear back-off instead of exponential
-	++m_retry_count;
-	m_send_timer.expires_from_now(milliseconds(250 * m_retry_count), ec);
-	m_send_timer.async_wait(bind(&natpmp::resend_request, self(), i, _1));
+	if (m_abort)
+	{
+		// when we're shutting down, ignore the
+		// responses and just remove all mappings
+		// immediately
+		m_currently_mapping = -1;
+		m.action = mapping_t::action_none;
+		try_next_mapping(i);
+	}
+	else
+	{
+		// linear back-off instead of exponential
+		++m_retry_count;
+		m_send_timer.expires_from_now(milliseconds(250 * m_retry_count), ec);
+		m_send_timer.async_wait(bind(&natpmp::resend_request, self(), i, _1));
+	}
 }
 
 void natpmp::resend_request(int i, error_code const& e)
@@ -330,6 +348,14 @@ void natpmp::on_reply(error_code const& e
 	m_socket.async_receive_from(asio::buffer(&m_response_buffer, 16)
 		, m_remote, bind(&natpmp::on_reply, self(), _1, _2));
 
+	// simulate packet loss
+/*
+	if ((rand() % 2) == 0)
+	{
+		log(" simulating drop");
+		return;
+	}
+*/
 	if (m_remote != m_nat_endpoint)
 	{
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
@@ -345,10 +371,6 @@ void natpmp::on_reply(error_code const& e
 	error_code ec;
 	m_send_timer.cancel(ec);
 
-	TORRENT_ASSERT(m_currently_mapping >= 0);
-	int i = m_currently_mapping;
-	mapping_t& m = m_mappings[i];
-
 	char* in = m_response_buffer;
 	int version = read_uint8(in);
 	int cmd = read_uint8(in);
@@ -360,6 +382,8 @@ void natpmp::on_reply(error_code const& e
 
 	(void)time; // to remove warning
 
+	int protocol = (cmd - 128 == 1)?udp:tcp;
+
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
 	m_log << time_now_string()
 		<< " <== port map ["
@@ -375,30 +399,36 @@ void natpmp::on_reply(error_code const& e
 	}
 #endif
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	if (private_port != m.local_port)
+	mapping_t* m = 0;
+	int index = -1;
+	for (std::vector<mapping_t>::iterator i = m_mappings.begin()
+		, end(m_mappings.end()); i != end; ++i)
 	{
-		m_log << "*** unexpected local port: " << private_port << std::endl;
+		if (private_port != i->local_port) continue;
+		if (protocol != i->protocol) continue;
+		m = &*i;
+		index = i - m_mappings.begin();
+		break;
 	}
-#endif
 
-#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
-	if (cmd != 128 + m.protocol)
+	if (m == 0)
 	{
-		m_log << "*** unexpected protocol: " << (cmd - 128) << std::endl;
-	}
+#if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
+		m_log << "*** not found in map table" << std::endl;
 #endif
+		return;
+	}
 
 	if (public_port == 0 || lifetime == 0)
 	{
 		// this means the mapping was
 		// successfully closed
-		m.protocol = none;
+		m->protocol = none;
 	}
 	else
 	{
-		m.expires = time_now() + seconds(int(lifetime * 0.7f));
-		m.external_port = public_port;
+		m->expires = time_now() + seconds(int(lifetime * 0.7f));
+		m->external_port = public_port;
 	}
 
 	if (result != 0)
@@ -416,19 +446,19 @@ void natpmp::on_reply(error_code const& e
 			case 4: errmsg << "Out of resources"; break;
 			case 5: errmsg << "Unsupported opcode"; break;
 		}
-		m.expires = time_now() + hours(2);
-		m_callback(i, 0, errmsg.str());
+		m->expires = time_now() + hours(2);
+		m_callback(index, 0, errmsg.str());
 	}
-	else if (m.action == mapping_t::action_add)
+	else if (m->action == mapping_t::action_add)
 	{
-		m_callback(i, m.external_port, "");
+		m_callback(index, m->external_port, "");
 	}
 
 	m_currently_mapping = -1;
-	m.action = mapping_t::action_none;
+	m->action = mapping_t::action_none;
 	m_send_timer.cancel(ec);
 	update_expiration_timer();
-	try_next_mapping(i);
+	try_next_mapping(index);
 }
 
 void natpmp::update_expiration_timer()
@@ -499,6 +529,11 @@ void natpmp::mapping_expired(error_code const& e, int i)
 void natpmp::close()
 {
 	mutex_t::scoped_lock l(m_mutex);
+	close_impl();
+}
+
+void natpmp::close_impl()
+{
 	m_abort = true;
 	error_code ec;
 #if defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)
@@ -522,6 +557,7 @@ void natpmp::close()
 		i->action = mapping_t::action_delete;
 	}
 	m_refresh_timer.cancel(ec);
+	m_currently_mapping = -1;
 	update_mapping(0);
 }
 
