@@ -75,18 +75,17 @@ namespace libtorrent
 #endif
 	}
 
-	void piece_picker::init(int blocks_per_piece, int total_num_blocks)
+	void piece_picker::init(int blocks_per_piece, int blocks_in_last_piece, int total_num_pieces)
 	{
 		TORRENT_ASSERT(blocks_per_piece > 0);
-		TORRENT_ASSERT(total_num_blocks >= 0);
+		TORRENT_ASSERT(total_num_pieces > 0);
 
 #ifdef TORRENT_PICKER_LOG
 		std::cerr << "piece_picker::init()" << std::endl;
 #endif
 		// allocate the piece_map to cover all pieces
 		// and make them invalid (as if we don't have a single piece)
-		m_piece_map.resize((total_num_blocks + blocks_per_piece-1) / blocks_per_piece
-			, piece_pos(0, 0));
+		m_piece_map.resize(total_num_pieces, piece_pos(0, 0));
 		m_reverse_cursor = int(m_piece_map.size());
 		m_cursor = 0;
 
@@ -117,7 +116,7 @@ namespace libtorrent
 		TORRENT_ASSERT(m_piece_map.size() < piece_pos::we_have_index);
 		
 		m_blocks_per_piece = blocks_per_piece;
-		m_blocks_in_last_piece = total_num_blocks % blocks_per_piece;
+		m_blocks_in_last_piece = blocks_in_last_piece;
 		if (m_blocks_in_last_piece == 0) m_blocks_in_last_piece = blocks_per_piece;
 
 		TORRENT_ASSERT(m_blocks_in_last_piece <= m_blocks_per_piece);
@@ -472,6 +471,7 @@ namespace libtorrent
 		TORRENT_ASSERT(m_seeds >= 0);
 		const float num_pieces = static_cast<float>(m_piece_map.size());
 
+		if (num_pieces == 0) return 1.0f;
 		int min_availability = piece_pos::max_peer_count;
 		// find the lowest availability count
 		// count the number of pieces that have that availability
@@ -1978,6 +1978,7 @@ namespace libtorrent
 			info.peer = peer;
 			info.num_peers = 1;
 			++dp.requested;
+			dp.last_request = time_now();
 		}
 		else
 		{
@@ -2002,6 +2003,7 @@ namespace libtorrent
 			}
 			++info.num_peers;
 			if (i->state == none) i->state = state;
+			i->last_request = time_now();
 		}
 		return true;
 	}
@@ -2024,6 +2026,23 @@ namespace libtorrent
 		return info.num_peers;
 	}
 	
+	ptime piece_picker::last_request(int piece) const
+	{
+		TORRENT_ASSERT(piece >= 0);
+		TORRENT_ASSERT(piece < (int)m_piece_map.size());
+
+		piece_pos const& p = m_piece_map[piece];
+		if (!p.downloading) return min_time();
+
+		std::vector<downloading_piece>::const_iterator i
+			= std::find_if(m_downloads.begin(), m_downloads.end(), has_index(piece));
+		TORRENT_ASSERT(i != m_downloads.end());
+		// just to play it safe
+		if (i == m_downloads.end()) return min_time();
+
+		return i->last_request;
+	}
+
 	void piece_picker::get_availability(std::vector<int>& avail) const
 	{
 		TORRENT_ASSERT(m_seeds >= 0);
