@@ -75,7 +75,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 //#define TORRENT_PARTIAL_HASH_LOG
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 #include <ios>
 #include <iostream>
 #include <iomanip>
@@ -137,112 +137,12 @@ namespace libtorrent
 }
 #endif
 
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION < 103400
-namespace
-{
-	using libtorrent::safe_convert;
-	using namespace boost::filesystem;
-	
-	// based on code from Boost.Fileystem
-	bool create_directories_win(const fs::path& ph)
-	{
-		if (ph.empty() || exists(ph))
-		{
-			if ( !ph.empty() && !is_directory(ph) )
-				boost::throw_exception( filesystem_error(
-					"boost::filesystem::create_directories",
-					ph, "path exists and is not a directory",
-					not_directory_error ) );
-			return false;
-		}
-
-		// First create branch, by calling ourself recursively
-		create_directories_win(ph.branch_path());
-		// Now that parent's path exists, create the directory
-		std::wstring wph(safe_convert(ph.native_directory_string()));
-		CreateDirectory(wph.c_str(), 0);
-		return true;
-	}
-
-	bool exists_win( const fs::path & ph )
-	{
-		std::wstring wpath(safe_convert(ph.string()));
-		if(::GetFileAttributes( wpath.c_str() ) == 0xFFFFFFFF)
-		{
-			UINT err = ::GetLastError();
-			if((err == ERROR_FILE_NOT_FOUND)
-				|| (err == ERROR_INVALID_PARAMETER)
-				|| (err == ERROR_NOT_READY)
-				|| (err == ERROR_PATH_NOT_FOUND)
-				|| (err == ERROR_INVALID_NAME)
-				|| (err == ERROR_BAD_NETPATH ))
-			return false; // GetFileAttributes failed because the path does not exist
-			// for any other error we assume the file does exist and fall through,
-			// this may not be the best policy though...  (JM 20040330)
-			return true;
-		}
-		return true;
-	}
-
-	boost::intmax_t file_size_win( const fs::path & ph )
-	{
-		std::wstring wpath(safe_convert(ph.string()));
-		// by now, intmax_t is 64-bits on all Windows compilers
-		WIN32_FILE_ATTRIBUTE_DATA fad;
-		if ( !::GetFileAttributesExW( wpath.c_str(),
-					::GetFileExInfoStandard, &fad ) )
-			boost::throw_exception( filesystem_error(
-				"boost::filesystem::file_size",
-				ph, detail::system_error_code() ) );
-		if ( (fad.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) !=0 )
-			boost::throw_exception( filesystem_error(
-				"boost::filesystem::file_size",
-				ph, "invalid: is a directory",
-				is_directory_error ) ); 
-		return (static_cast<boost::intmax_t>(fad.nFileSizeHigh)
-			<< (sizeof(fad.nFileSizeLow)*8))
-			+ fad.nFileSizeLow;
-	}
-	
-	std::time_t last_write_time_win( const fs::path & ph )
-	{
-		struct _stat path_stat;
-		std::wstring wph(safe_convert(ph.native_file_string()));
-		if ( ::_wstat( wph.c_str(), &path_stat ) != 0 )
-			boost::throw_exception( filesystem_error(
-			"boost::filesystem::last_write_time",
-			ph, detail::system_error_code() ) );
-		return path_stat.st_mtime;
-	}
-
-	void rename_win( const fs::path & old_path,
-		const fs::path & new_path )
-	{
-		std::wstring wold_path(safe_convert(old_path.string()));
-		std::wstring wnew_path(safe_convert(new_path.string()));
-		if ( !::MoveFile( wold_path.c_str(), wnew_path.c_str() ) )
-		boost::throw_exception( filesystem_error(
-			"boost::filesystem::rename",
-			old_path, new_path, detail::system_error_code() ) );
-	}
-
-} // anonymous namespace
-
-#endif
-
-#if BOOST_VERSION < 103200
-bool operator<(fs::path const& lhs, fs::path const& rhs)
-{
-	return lhs.string() < rhs.string();
-}
-#endif
-
 namespace fs = boost::filesystem;
 using boost::bind;
 using namespace ::boost::multi_index;
 using boost::multi_index::multi_index_container;
 
-#if !defined(NDEBUG) && defined(TORRENT_STORAGE_DEBUG)
+#if defined TORRENT_DEBUG && defined TORRENT_STORAGE_DEBUG
 namespace
 {
 	using namespace libtorrent;
@@ -272,9 +172,9 @@ namespace libtorrent
 			for (basic_directory_iterator<Path> i(old_path), end; i != end; ++i)
 			{
 #if BOOST_VERSION < 103600
-				recursive_copy(i->path(), new_path / i->leaf(), ec);
+				recursive_copy(i->path(), new_path / i->path().leaf(), ec);
 #else
-				recursive_copy(i->path(), new_path / i->filename(), ec);
+				recursive_copy(i->path(), new_path / i->path().filename(), ec);
 #endif
 				if (ec) return;
 			}
@@ -284,8 +184,19 @@ namespace libtorrent
 			copy_file(old_path, new_path);
 		}
 #ifndef BOOST_NO_EXCEPTIONS
-		} catch (std::exception& e) { ec = error_code(errno, get_posix_category()); }
-#endif
+		}
+#if BOOST_VERSION >= 103500
+		catch (boost::system::system_error& e)
+		{
+			ec = e.code();
+		}
+#else
+		catch (boost::filesystem::filesystem_error& e)
+		{
+			ec = error_code(e.system_error(), get_system_category());
+		}
+#endif // BOOST_VERSION
+#endif // BOOST_NO_EXCEPTIONS
 	}
 
 	template <class Path>
@@ -390,7 +301,7 @@ namespace libtorrent
 				|| (!compact_mode && size < s->first))
 			{
 				if (error) *error = "filesize mismatch for file '"
-					+ i->path.native_file_string()
+					+ i->path.external_file_string()
 					+ "', size: " + boost::lexical_cast<std::string>(size)
 					+ ", expected to be " + boost::lexical_cast<std::string>(s->first)
 					+ " bytes";
@@ -400,10 +311,10 @@ namespace libtorrent
 			// in sparse mode, allow the files to be more recent
 			// than the resume data, but only by 5 minutes
 			if ((compact_mode && (time > s->second + 1 || time < s->second - 1)) ||
-				(!compact_mode && (time > s->second + 5 * 60) || time < s->second - 1))
+				(!compact_mode && (time > s->second + 5 * 60 || time < s->second - 1)))
 			{
 				if (error) *error = "timestamp mismatch for file '"
-					+ i->path.native_file_string()
+					+ i->path.external_file_string()
 					+ "', modification date: " + boost::lexical_cast<std::string>(time)
 					+ ", expected to have modification date "
 					+ boost::lexical_cast<std::string>(s->second);
@@ -416,15 +327,18 @@ namespace libtorrent
 	class storage : public storage_interface, boost::noncopyable
 	{
 	public:
-		storage(file_storage const& fs, fs::path const& path, file_pool& fp)
+		storage(file_storage const& fs, file_storage const* mapped, fs::path const& path, file_pool& fp)
 			: m_files(fs)
 			, m_pool(fp)
 		{
+			if (mapped) m_mapped_files.reset(new file_storage(*mapped));
+
 			TORRENT_ASSERT(m_files.begin() != m_files.end());
 			m_save_path = fs::complete(path);
 			TORRENT_ASSERT(m_save_path.is_complete());
 		}
 
+		bool has_any_file();
 		bool rename_file(int index, std::string const& new_filename);
 		bool release_files();
 		bool delete_files();
@@ -439,6 +353,7 @@ namespace libtorrent
 		bool write_resume_data(entry& rd) const;
 		sha1_hash hash_for_slot(int slot, partial_hash& ph, int piece_size);
 
+		void delete_one_file(std::string const& p);
 		int read_impl(char* buf, int slot, int offset, int size, bool fill_zero);
 
 		~storage()
@@ -463,7 +378,7 @@ namespace libtorrent
 	sha1_hash storage::hash_for_slot(int slot, partial_hash& ph, int piece_size)
 	{
 		TORRENT_ASSERT(!error());
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		hasher partial;
 		hasher whole;
 		int slot_size1 = piece_size;
@@ -484,7 +399,7 @@ namespace libtorrent
 			if (error()) return sha1_hash(0);
 			ph.h.update(&m_scratch_buffer[0], slot_size);
 		}
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		sha1_hash ret = ph.h.final();
 		TORRENT_ASSERT(ret == whole.final());
 		return ret;
@@ -507,10 +422,7 @@ namespace libtorrent
 			{
 				last_path = dir;
 
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION < 103400
-				if (!exists_win(last_path))
-					create_directories_win(last_path);
-#elif TORRENT_USE_WPATH
+#if TORRENT_USE_WPATH
 				fs::wpath wp = safe_convert(last_path.string());
 				if (!exists(wp))
 					create_directories(wp);
@@ -548,13 +460,21 @@ namespace libtorrent
 			}
 #ifndef BOOST_NO_EXCEPTIONS
 			}
-			catch (std::exception& e)
+#if BOOST_VERSION >= 103500
+			catch (boost::system::system_error& e)
 			{
-				set_error(m_save_path / file_iter->path
-					, error_code(errno, get_posix_category()));
+				set_error(m_save_path / file_iter->path, e.code());
 				return true;
 			}
-#endif
+#else
+			catch (boost::filesystem::filesystem_error& e)
+			{
+				set_error(m_save_path / file_iter->path
+					, error_code(e.system_error(), get_system_category()));
+				return true;
+			}
+#endif // BOOST_VERSION
+#endif // BOOST_NO_EXCEPTIONS
 		}
 		std::vector<boost::uint8_t>().swap(m_file_priority);
 		// close files that were opened in write mode
@@ -562,6 +482,47 @@ namespace libtorrent
 		return false;
 	}
 
+	bool storage::has_any_file()
+	{
+		file_storage::iterator i = files().begin();
+		file_storage::iterator end = files().end();
+
+		for (; i != end; ++i)
+		{
+			bool file_exists = false;
+			fs::path f = m_save_path / i->path;
+#ifndef BOOST_NO_EXCEPTIONS
+			try
+			{
+#endif
+#if TORRENT_USE_WPATH
+				fs::wpath wf = safe_convert(f.string());
+				file_exists = exists(wf);
+#else
+				file_exists = exists(f);
+#endif
+#ifndef BOOST_NO_EXCEPTIONS
+			}
+#if BOOST_VERSION >= 103500
+			catch (boost::system::system_error& e)
+			{
+				set_error(f, e.code());
+				return false;
+			}
+#else
+			catch (boost::filesystem::filesystem_error& e)
+			{
+				set_error(f, error_code(e.system_error(), get_system_category()));
+				return false;
+			}
+#endif // BOOST_VERSION
+#endif // BOOST_NO_EXCEPTIONS
+			if (file_exists && i->size > 0)
+				return true;
+		}
+		return false;
+	}
+
 	bool storage::rename_file(int index, std::string const& new_filename)
 	{
 		if (index < 0 || index >= m_files.num_files()) return true;
@@ -605,12 +566,14 @@ namespace libtorrent
 			set_error(old_name, e.code());
 			return true;
 		}
-#endif
-		catch (std::exception& e)
+#else
+		catch (boost::filesystem::filesystem_error& e)
 		{
-			set_error(old_name, error_code(errno, get_posix_category()));
+			set_error(old_name, error_code(e.system_error()
+				, get_system_category()));
 			return true;
 		}
+#endif // BOOST_VERSION
 #endif
 		return false;
 	}
@@ -622,15 +585,43 @@ namespace libtorrent
 		return false;
 	}
 
+	void storage::delete_one_file(std::string const& p)
+	{
+#if TORRENT_USE_WPATH
+#ifndef BOOST_NO_EXCEPTIONS
+		try
+#endif
+		{ fs::remove(safe_convert(p)); }
+#ifndef BOOST_NO_EXCEPTIONS
+#if BOOST_VERSION >= 103500
+		catch (boost::system::system_error& e)
+		{
+			// no such file or directory is not an error
+			if (e.code() != make_error_code(boost::system::errc::no_such_file_or_directory))
+				set_error(p, e.code());
+		}
+#else
+		catch (boost::filesystem::filesystem_error& e)
+		{
+			set_error(p, error_code(e.system_error(), get_system_category()));
+		}
+#endif // BOOST_VERSION
+#endif // BOOST_NO_EXCEPTIONS
+#else // TORRENT_USE_WPATH
+		// no such file or directory is not an error
+		if (std::remove(p.c_str()) != 0 && errno != ENOENT)
+		{
+			set_error(p, error_code(errno, get_posix_category()));
+		}
+#endif
+	}
+
 	bool storage::delete_files()
 	{
 		// make sure we don't have the files open
 		m_pool.release(this);
 		buffer().swap(m_scratch_buffer);
 
-		int error = 0;
-		std::string error_file;
-
 		// delete the files from disk
 		std::set<std::string> directories;
 		typedef std::set<std::string>::iterator iter_t;
@@ -643,24 +634,10 @@ namespace libtorrent
 			ret.second = true;
 			while (ret.second && !bp.empty())
 			{
-				std::pair<iter_t, bool> ret = directories.insert((m_save_path / bp).string());
+				ret = directories.insert((m_save_path / bp).string());
 				bp = bp.branch_path();
 			}
-#if TORRENT_USE_WPATH
-			try
-			{ fs::remove(safe_convert(p)); }
-			catch (std::exception& e)
-			{
-				error = errno;
-				error_file = p;
-			}
-#else
-			if (std::remove(p.c_str()) != 0 && errno != ENOENT)
-			{
-				error = errno;
-				error_file = p;
-			}
-#endif
+			delete_one_file(p);
 		}
 
 		// remove the directories. Reverse order to delete
@@ -669,29 +646,10 @@ namespace libtorrent
 		for (std::set<std::string>::reverse_iterator i = directories.rbegin()
 			, end(directories.rend()); i != end; ++i)
 		{
-#if TORRENT_USE_WPATH
-			try
-			{ fs::remove(safe_convert(*i)); }
-			catch (std::exception& e)
-			{
-				error = errno;
-				error_file = *i;
-			}
-#else
-			if (std::remove(i->c_str()) != 0 && errno != ENOENT)
-			{
-				error = errno;
-				error_file = *i;
-			}
-#endif
+			delete_one_file(*i);
 		}
 
-		if (error)
-		{
-			m_error = error_code(error, get_posix_category());
-			m_error_file.swap(error_file);
-			return true;
-		}
+		if (error()) return true;
 		return false;
 	}
 
@@ -712,21 +670,23 @@ namespace libtorrent
 			fl.push_back(entry(p));
 		}
 		
-		if (m_mapped_files)
-		{
-			entry::list_type& fl = rd["mapped_files"].list();
-			for (file_storage::iterator i = m_mapped_files->begin()
-				, end(m_mapped_files->end()); i != end; ++i)
-			{
-				fl.push_back(i->path.string());
-			}
-		}
-
 		return false;
 	}
 
 	bool storage::verify_resume_data(lazy_entry const& rd, std::string& error)
 	{
+		lazy_entry const* mapped_files = rd.dict_find_list("mapped_files");
+		if (mapped_files && mapped_files->list_size() == m_files.num_files())
+		{
+			m_mapped_files.reset(new file_storage(m_files));
+			for (int i = 0; i < m_files.num_files(); ++i)
+			{
+				std::string new_filename = mapped_files->list_string_value_at(i);
+				if (new_filename.empty()) continue;
+				m_mapped_files->rename_file(i, new_filename);
+			}
+		}
+		
 		lazy_entry const* file_priority = rd.dict_find_list("file_priority");
 		if (file_priority && file_priority->list_size()
 			== files().num_files())
@@ -821,7 +781,7 @@ namespace libtorrent
 			{
 				if (i->size != fs->first)
 				{
-					error = "file size for '" + i->path.native_file_string()
+					error = "file size for '" + i->path.external_file_string()
 						+ "' was expected to be "
 						+ boost::lexical_cast<std::string>(i->size) + " bytes";
 					return false;
@@ -846,13 +806,7 @@ namespace libtorrent
 
 		save_path = complete(save_path);
 
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION < 103400
-		std::wstring wsave_path(safe_convert(save_path.native_file_string()));
-		if (!exists_win(save_path))
-			CreateDirectory(wsave_path.c_str(), 0);
-		else if ((GetFileAttributes(wsave_path.c_str()) & FILE_ATTRIBUTE_DIRECTORY) == 0)
-			return false;
-#elif TORRENT_USE_WPATH
+#if TORRENT_USE_WPATH
 		fs::wpath wp = safe_convert(save_path.string());
 		if (!exists(wp))
 			create_directory(wp);
@@ -867,44 +821,59 @@ namespace libtorrent
 
 		m_pool.release(this);
 
+		bool ret = true;
+		std::set<std::string> to_move;
+		file_storage const& f = files();
+
+		for (file_storage::iterator i = f.begin()
+			, end(f.end()); i != end; ++i)
+		{
+			to_move.insert(to_move.begin(), *i->path.begin());
+		}
+
+		for (std::set<std::string>::const_iterator i = to_move.begin()
+			, end(to_move.end()); i != end; ++i)
+		{
+			
 #if TORRENT_USE_WPATH
-		old_path = safe_convert((m_save_path / files().name()).string());
-		new_path = safe_convert((save_path / files().name()).string());
+			old_path = safe_convert((m_save_path / *i).string());
+			new_path = safe_convert((save_path / *i).string());
 #else
-		old_path = m_save_path / files().name();
-		new_path = save_path / files().name();
+			old_path = m_save_path / *i;
+			new_path = save_path / *i;
 #endif
 
 #ifndef BOOST_NO_EXCEPTIONS
-		try
-		{
-#endif
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION < 103400
-			rename_win(old_path, new_path);
-#else
-			rename(old_path, new_path);
-#endif
-			m_save_path = save_path;
-			return true;
-#ifndef BOOST_NO_EXCEPTIONS
-		}
-		catch (std::exception& e)
-		{
-			error_code ec;
-			recursive_copy(old_path, new_path, ec);
-			if (ec)
+			try
 			{
-				set_error(m_save_path / files().name(), ec);
-				return true;
-			}
-			m_save_path = save_path;
-			recursive_remove(old_path);
-		}
 #endif
-		return false;
+				if (exists(old_path))
+					rename(old_path, new_path);
+#ifndef BOOST_NO_EXCEPTIONS
+			}
+			catch (std::exception& e)
+			{
+				error_code ec;
+				recursive_copy(old_path, new_path, ec);
+				if (ec)
+				{
+					set_error(m_save_path / files().name(), ec);
+					ret = false;
+				}
+				else
+				{
+					recursive_remove(old_path);
+				}
+			}
+#endif
+		}
+
+		if (ret) m_save_path = save_path;
+
+		return ret;
 	}
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 /*
 	void storage::shuffle()
 	{
@@ -999,7 +968,7 @@ namespace libtorrent
 		TORRENT_ASSERT(offset < m_files.piece_size(slot));
 		TORRENT_ASSERT(size > 0);
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		std::vector<file_slice> slices
 			= files().map_block(slot, offset, size);
 		TORRENT_ASSERT(!slices.empty());
@@ -1046,7 +1015,7 @@ namespace libtorrent
 			return size;
 		}
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		size_type in_tell = in->tell(ec);
 		TORRENT_ASSERT(in_tell == file_offset + file_iter->file_base && !ec);
 #endif
@@ -1061,7 +1030,7 @@ namespace libtorrent
 
 		size_type result = left_to_read;
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		int counter = 0;
 #endif
 
@@ -1073,7 +1042,7 @@ namespace libtorrent
 
 			if (read_bytes > 0)
 			{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				TORRENT_ASSERT(int(slices.size()) > counter);
 				size_type slice_size = slices[counter].size;
 				TORRENT_ASSERT(slice_size == read_bytes);
@@ -1083,15 +1052,26 @@ namespace libtorrent
 
 				int actual_read = int(in->read(buf + buf_pos, read_bytes, ec));
 
-				if (read_bytes != actual_read || ec)
+				if (ec)
+				{
+					set_error(m_save_path / file_iter->path, ec);
+					return -1;
+				}
+
+				if (read_bytes != actual_read)
 				{
 					// the file was not big enough
-					if (actual_read > 0) buf_pos += actual_read;
 					if (!fill_zero)
 					{
+#ifdef TORRENT_WINDOWS
+						ec = error_code(ERROR_READ_FAULT, get_system_category());
+#else
+						ec = error_code(EIO, get_posix_category());
+#endif
 						set_error(m_save_path / file_iter->path, ec);
-						return -1;
+						return actual_read;
 					}
+					if (actual_read > 0) buf_pos += actual_read;
 					std::memset(buf + buf_pos, 0, size - buf_pos);
 					return size;
 				}
@@ -1105,7 +1085,7 @@ namespace libtorrent
 			if (left_to_read > 0)
 			{
 				++file_iter;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				// empty files are not returned by map_block, so if
 				// this file was empty, don't increment the slice counter
 				if (read_bytes > 0) ++counter;
@@ -1148,7 +1128,7 @@ namespace libtorrent
 		TORRENT_ASSERT(offset >= 0);
 		TORRENT_ASSERT(size > 0);
 
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		std::vector<file_slice> slices
 			= files().map_block(slot, offset, size);
 		TORRENT_ASSERT(!slices.empty());
@@ -1200,7 +1180,7 @@ namespace libtorrent
 		TORRENT_ASSERT(left_to_write >= 0);
 
 		int buf_pos = 0;
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		int counter = 0;
 #endif
 		while (left_to_write > 0)
@@ -1224,12 +1204,24 @@ namespace libtorrent
 				error_code ec;
 				size_type written = out->write(buf + buf_pos, write_bytes, ec);
 
-				if (written != write_bytes || ec)
+				if (ec)
 				{
 					set_error(m_save_path / file_iter->path, ec);
 					return -1;
 				}
 
+				if (write_bytes != written)
+				{
+					// the file was not big enough
+#ifdef TORRENT_WINDOWS
+					ec = error_code(ERROR_READ_FAULT, get_system_category());
+#else
+					ec = error_code(EIO, get_posix_category());
+#endif
+					set_error(m_save_path / file_iter->path, ec);
+					return written;
+				}
+
 				left_to_write -= write_bytes;
 				buf_pos += write_bytes;
 				TORRENT_ASSERT(buf_pos >= 0);
@@ -1239,7 +1231,7 @@ namespace libtorrent
 
 			if (left_to_write > 0)
 			{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				if (write_bytes > 0) ++counter;
 #endif
 				++file_iter;
@@ -1270,9 +1262,9 @@ namespace libtorrent
 	}
 
 	storage_interface* default_storage_constructor(file_storage const& fs
-		, fs::path const& path, file_pool& fp)
+		, file_storage const* mapped, fs::path const& path, file_pool& fp)
 	{
-		return new storage(fs, path, fp);
+		return new storage(fs, mapped, path, fp);
 	}
 
 	// -- piece_manager -----------------------------------------------------
@@ -1287,7 +1279,8 @@ namespace libtorrent
 		, storage_mode_t sm)
 		: m_info(info)
 		, m_files(m_info->files())
-		, m_storage(sc(m_files, save_path, fp))
+		, m_storage(sc(m_info->orig_files(), &m_info->files() != &m_info->orig_files()
+			? &m_info->files() : 0, save_path, fp))
 		, m_storage_mode(sm)
 		, m_save_path(complete(save_path))
 		, m_state(state_none)
@@ -1331,6 +1324,11 @@ namespace libtorrent
 		m_io_thread.add_job(j, handler);
 	}
 
+	void piece_manager::abort_disk_io()
+	{
+		m_io_thread.stop(this);
+	}
+
 	void piece_manager::async_delete_files(
 		boost::function<void(int, disk_io_job const&)> const& handler)
 	{
@@ -1398,7 +1396,7 @@ namespace libtorrent
 		// since that is the size of the pool allocator's buffers
 		TORRENT_ASSERT(r.length <= 16 * 1024);
 		m_io_thread.add_job(j, handler);
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 		boost::recursive_mutex::scoped_lock l(m_mutex);
 		// if this assert is hit, it suggests
 		// that check_files was not successful
@@ -1569,7 +1567,7 @@ namespace libtorrent
 			std::map<int, partial_hash>::iterator i = m_piece_hasher.find(piece_index);
 			if (i != m_piece_hasher.end())
 			{
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 				TORRENT_ASSERT(i->second.offset > 0);
 				int hash_offset = i->second.offset;
 				TORRENT_ASSERT(offset >= hash_offset);
@@ -1748,53 +1746,27 @@ namespace libtorrent
 
 	int piece_manager::check_no_fastresume(std::string& error)
 	{
-		file_storage::iterator i = m_files.begin();
-		file_storage::iterator end = m_files.end();
+		bool has_files = m_storage->has_any_file();
 
-		for (; i != end; ++i)
+		if (m_storage->error())
+			return fatal_disk_error;
+
+		if (has_files)
 		{
-			bool file_exists = false;
-			fs::path f = m_save_path / i->path;
-#ifndef BOOST_NO_EXCEPTIONS
-			try
+			m_state = state_full_check;
+			m_piece_to_slot.clear();
+			m_piece_to_slot.resize(m_files.num_pieces(), has_no_slot);
+			m_slot_to_piece.clear();
+			m_slot_to_piece.resize(m_files.num_pieces(), unallocated);
+			if (m_storage_mode == storage_mode_compact)
 			{
-#endif
-#if defined(_WIN32) && defined(UNICODE) && BOOST_VERSION < 103400
-				file_exists = exists_win(f);
-#elif TORRENT_USE_WPATH
-				fs::wpath wf = safe_convert(f.string());
-				file_exists = exists(wf);
-#else
-				file_exists = exists(f);
-#endif
-#ifndef BOOST_NO_EXCEPTIONS
-			}
-			catch (std::exception& e)
-			{
-				error = f.string();
-				error += ": ";
-				error += e.what();
-				TORRENT_ASSERT(!error.empty());
-				return fatal_disk_error;
-			}
-#endif
-			if (file_exists && i->size > 0)
-			{
-				m_state = state_full_check;
-				m_piece_to_slot.clear();
-				m_piece_to_slot.resize(m_files.num_pieces(), has_no_slot);
-				m_slot_to_piece.clear();
-				m_slot_to_piece.resize(m_files.num_pieces(), unallocated);
-				if (m_storage_mode == storage_mode_compact)
-				{
-					m_unallocated_slots.clear();
-					m_free_slots.clear();
-				}
-				TORRENT_ASSERT(int(m_piece_to_slot.size()) == m_files.num_pieces());
-				return need_full_check;
+				m_unallocated_slots.clear();
+				m_free_slots.clear();
 			}
+			TORRENT_ASSERT(int(m_piece_to_slot.size()) == m_files.num_pieces());
+			return need_full_check;
 		}
-	
+
 		if (m_storage_mode == storage_mode_compact)
 		{
 			// in compact mode without checking, we need to
@@ -1847,6 +1819,8 @@ namespace libtorrent
 		INVARIANT_CHECK;
 
 		TORRENT_ASSERT(m_files.piece_length() > 0);
+		
+		m_current_slot = 0;
 
 		// if we don't have any resume data, return
 		if (rd.type() == lazy_entry::none_t) return check_no_fastresume(error);
@@ -2052,6 +2026,8 @@ namespace libtorrent
 	// is finished
 	int piece_manager::check_files(int& current_slot, int& have_piece, std::string& error)
 	{
+		if (m_state == state_none) return check_no_fastresume(error);
+
 		TORRENT_ASSERT(int(m_piece_to_slot.size()) == m_files.num_pieces());
 
 		current_slot = m_current_slot;
@@ -2272,7 +2248,10 @@ namespace libtorrent
 #else
 				&& m_storage->error() != error_code(ENOENT, get_posix_category()))
 #endif
+			{
+				m_piece_data.clear();
 				return -1;
+			}
 			return 1;
 		}
 
@@ -2574,7 +2553,7 @@ namespace libtorrent
 			&& m_slot_to_piece[piece_index] >= 0)
 		{
 
-#if !defined(NDEBUG) && defined(TORRENT_STORAGE_DEBUG)
+#if defined TORRENT_DEBUG && defined TORRENT_STORAGE_DEBUG
 			std::stringstream s;
 
 			s << "there is another piece at our slot, swapping..";
@@ -2609,7 +2588,7 @@ namespace libtorrent
 
 			slot_index = piece_index;
 
-#if !defined(NDEBUG) && defined(TORRENT_STORAGE_DEBUG)
+#if defined TORRENT_DEBUG && defined TORRENT_STORAGE_DEBUG
 			debug_log();
 #endif
 		}
@@ -2677,7 +2656,7 @@ namespace libtorrent
 		return m_slot_to_piece[slot];
 	}
 		
-#ifndef NDEBUG
+#ifdef TORRENT_DEBUG
 	void piece_manager::check_invariant() const
 	{
 		boost::recursive_mutex::scoped_lock lock(m_mutex);
