@@ -67,6 +67,7 @@ namespace libtorrent
 		, policy::peer* peerinfo)
 		: peer_connection(ses, t, s, remote, peerinfo)
 		, m_url(url)
+		, m_original_url(url)
 		, m_first_request(true)
 		, m_range_pos(0)
 	{
@@ -252,7 +253,7 @@ namespace libtorrent
 			request += "\r\nRange: bytes=";
 			request += to_string(size_type(r.piece) * info.piece_length() + r.start).elems;
 			request += "-";
-			request += to_string(r.piece * info.piece_length() + r.start + r.length - 1).elems;
+			request += to_string(size_type(r.piece) * info.piece_length() + r.start + r.length - 1).elems;
 			if (m_first_request || using_proxy)
 				request += "\r\nConnection: keep-alive";
 			request += "\r\n\r\n";
@@ -402,9 +403,9 @@ namespace libtorrent
 					if (m_parser.status_code() == 503)
 					{
 						// temporarily unavailable, retry later
-						t->retry_url_seed(m_url);
+						t->retry_url_seed(m_original_url);
 					}
-					t->remove_url_seed(m_url);
+					t->remove_url_seed(m_original_url);
 					std::string error_msg = to_string(m_parser.status_code()).elems
 						+ (" " + m_parser.message());
 					if (m_ses.m_alerts.should_post<url_seed_alert>())
@@ -447,7 +448,7 @@ namespace libtorrent
 					if (location.empty())
 					{
 						// we should not try this server again.
-						t->remove_url_seed(m_url);
+						t->remove_url_seed(m_original_url);
 						disconnect("got HTTP redirection status without location header", 2);
 						return;
 					}
@@ -468,7 +469,7 @@ namespace libtorrent
 						size_t i = location.rfind(path);
 						if (i == std::string::npos)
 						{
-							t->remove_url_seed(m_url);
+							t->remove_url_seed(m_original_url);
 							std::stringstream msg;
 							msg << "got invalid HTTP redirection location (\"" << location << "\") "
 								"expected it to end with: " << path;
@@ -478,7 +479,7 @@ namespace libtorrent
 						location.resize(i);
 					}
 					t->add_url_seed(location);
-					t->remove_url_seed(m_url);
+					t->remove_url_seed(m_original_url);
 					std::stringstream msg;
 					msg << "redirecting to \"" << location << "\"";
 					disconnect(msg.str().c_str());
@@ -516,7 +517,7 @@ namespace libtorrent
 				if (!range_str)
 				{
 					// we should not try this server again.
-					t->remove_url_seed(m_url);
+					t->remove_url_seed(m_original_url);
 					std::stringstream msg;
 					msg << "invalid range in HTTP response: " << range_str.str();
 					disconnect(msg.str().c_str(), 2);
@@ -532,7 +533,7 @@ namespace libtorrent
 				if (range_end == -1)
 				{
 					// we should not try this server again.
-					t->remove_url_seed(m_url);
+					t->remove_url_seed(m_original_url);
 					disconnect("no content-length in HTTP response", 2);
 					return;
 				}
