@@ -348,7 +348,7 @@ struct parse_state
 namespace libtorrent
 {
 	// defined in torrent_info.cpp
-	bool verify_encoding(std::string& target);
+	bool verify_encoding(std::string& target, bool path = true);
 }
 
 void find_control_url(int type, char const* string, parse_state& state);
@@ -450,9 +450,9 @@ int test_main()
 	test = "\b?filename=4";
 	TEST_CHECK(!verify_encoding(test));
 #ifdef TORRENT_WINDOWS
-	TEST_CHECK(test == "..filename=4");
+	TEST_CHECK(test == "__filename=4");
 #else
-	TEST_CHECK(test == ".?filename=4");
+	TEST_CHECK(test == "_?filename=4");
 #endif
 
 	test = "filename=4";
