@@ -1,3 +1,35 @@
+/*
+
+Copyright (c) 2008, Arvid Norberg
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
+
+    * Redistributions of source code must retain the above copyright
+      notice, this list of conditions and the following disclaimer.
+    * Redistributions in binary form must reproduce the above copyright
+      notice, this list of conditions and the following disclaimer in
+      the documentation and/or other materials provided with the distribution.
+    * Neither the name of the author nor the names of its
+      contributors may be used to endorse or promote products derived
+      from this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+POSSIBILITY OF SUCH DAMAGE.
+
+*/
+
 #include "libtorrent/session.hpp"
 #include "libtorrent/session_settings.hpp"
 #include "libtorrent/hasher.hpp"
@@ -56,9 +88,9 @@ int test_main()
 	{
 		file_storage fs;
 		size_type file_size = 1 * 1024 * 1024 * 1024;
-		fs.add_file("test_torrent/tmp1", file_size);
-		fs.add_file("test_torrent/tmp2", file_size);
-		fs.add_file("test_torrent/tmp3", file_size);
+		fs.add_file("test_torrent_dir2/tmp1", file_size);
+		fs.add_file("test_torrent_dir2/tmp2", file_size);
+		fs.add_file("test_torrent_dir2/tmp3", file_size);
 		libtorrent::create_torrent t(fs, 4 * 1024 * 1024);
 		t.add_tracker("http://non-existing.com/announce");
 
@@ -83,7 +115,7 @@ int test_main()
 	{
 		file_storage fs;
 
-		fs.add_file("test_torrent/tmp1", 0);
+		fs.add_file("test_torrent_dir2/tmp1", 0);
 		libtorrent::create_torrent t(fs, 4 * 1024 * 1024);
 		t.add_tracker("http://non-existing.com/announce");
 
