@@ -87,6 +87,8 @@ void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy)
 	float rate_sum = 0.f;
 	float ses_rate_sum = 0.f;
 
+	cache_status cs;
+
 	for (int i = 0; i < 30; ++i)
 	{
 		torrent_status s = th.status();
@@ -99,6 +101,7 @@ void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy)
 			<< std::endl;
 		rate_sum += s.download_payload_rate;
 		ses_rate_sum += ss.payload_download_rate;
+		cs = ses.get_cache_status();
 
 		print_alerts(ses, "ses");
 
@@ -111,6 +114,8 @@ void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy)
 		test_sleep(1000);
 	}
 
+	TEST_CHECK(cs.cache_size == 0);
+
 	std::cerr << "total_size: " << total_size
 		<< " rate_sum: " << rate_sum
 		<< " session_rate_sum: " << ses_rate_sum
@@ -162,7 +167,7 @@ int test_main()
 
 	file_pool fp;
 	boost::scoped_ptr<storage_interface> s(default_storage_constructor(
-		fs, ".", fp));
+		fs, 0, ".", fp));
 
 	for (int i = 0; i < num; ++i)
 	{
